@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2008 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2009 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -32,8 +32,6 @@
 #include "tbb_stddef.h"
 #include <algorithm>
 #include <iterator>
-#include <memory>
-#include <limits>
 #include <new>
 #include <cstring>
 #include "atomic.h"
@@ -42,23 +40,36 @@
 
 #include "tbb_machine.h"
 
-#if defined(_MSC_VER) && defined(_Wp64)
+#if _MSC_VER==1500 && !__INTEL_COMPILER
+    // VS2008/VC9 seems to have an issue; limits pull in math.h
+    #pragma warning( push )
+    #pragma warning( disable: 4985 )
+#endif
+#include <limits> /* std::numeric_limits */
+#if _MSC_VER==1500 && !__INTEL_COMPILER
+    #pragma warning( pop )
+#endif
+
+#if defined(_MSC_VER) && !defined(__INTEL_COMPILER) && defined(_Wp64)
     // Workaround for overzealous compiler warnings in /Wp64 mode
     #pragma warning (push)
     #pragma warning (disable: 4267)
-#endif /* _MSC_VER && _Wp64 */
+#endif
 
 namespace tbb {
 
 template<typename T, class A = cache_aligned_allocator<T> >
 class concurrent_vector;
 
-//! Bad allocation marker
-#define __TBB_BAD_ALLOC reinterpret_cast<void*>(63)
 
 //! @cond INTERNAL
 namespace internal {
 
+    //! Bad allocation marker
+    static void *const vector_allocation_error_flag = reinterpret_cast<void*>(size_t(63));
+    //! Routine that loads pointer from location pointed to by src without any fence, without causing ITT to report a race.
+    void* __TBB_EXPORTED_FUNC itt_load_pointer_v3( const void* src );
+
     //! Base class of concurrent vector implementation.
     /** @ingroup containers */
     class concurrent_vector_base_v3 {
@@ -80,11 +91,11 @@ namespace internal {
         // Segment pointer. Can be zero-initialized
         struct segment_t {
             void* array;
-#if TBB_DO_ASSERT
+#if TBB_USE_ASSERT
             ~segment_t() {
-                __TBB_ASSERT( array <= __TBB_BAD_ALLOC, "should have been freed by clear" );
+                __TBB_ASSERT( array <= internal::vector_allocation_error_flag, "should have been freed by clear" );
             }
-#endif /* TBB_DO_ASSERT */
+#endif /* TBB_USE_ASSERT */
         };
  
         // Data fields
@@ -113,7 +124,7 @@ namespace internal {
                 my_storage[i].array = NULL;
             my_segment = my_storage;
         }
-        ~concurrent_vector_base_v3();
+        __TBB_EXPORTED_METHOD ~concurrent_vector_base_v3();
 
         static segment_index_t segment_index_of( size_type index ) {
             return segment_index_t( __TBB_Log2( index|1 ) );
@@ -134,10 +145,10 @@ namespace internal {
         }
 
         //! An operation on an n-element array starting at begin.
-        typedef void(*internal_array_op1)(void* begin, size_type n );
+        typedef void (__TBB_EXPORTED_FUNC *internal_array_op1)(void* begin, size_type n );
 
         //! An operation on n-element destination array and n-element source array.
-        typedef void(*internal_array_op2)(void* dst, const void* src, size_type n );
+        typedef void (__TBB_EXPORTED_FUNC *internal_array_op2)(void* dst, const void* src, size_type n );
 
         //! Internal structure for compact()
         struct internal_segments_table {
@@ -145,20 +156,25 @@ namespace internal {
             void* table[pointers_per_long_table];
         };
 
-        void internal_reserve( size_type n, size_type element_size, size_type max_size );
-        size_type internal_capacity() const;
-        void internal_grow_to_at_least( size_type new_size, size_type element_size, internal_array_op2 init, const void *src );
+        void __TBB_EXPORTED_METHOD internal_reserve( size_type n, size_type element_size, size_type max_size );
+        size_type __TBB_EXPORTED_METHOD internal_capacity() const;
         void internal_grow( size_type start, size_type finish, size_type element_size, internal_array_op2 init, const void *src );
-        size_type internal_grow_by( size_type delta, size_type element_size, internal_array_op2 init, const void *src );
-        void* internal_push_back( size_type element_size, size_type& index );
-        segment_index_t internal_clear( internal_array_op1 destroy );
-        void* internal_compact( size_type element_size, void *table, internal_array_op1 destroy, internal_array_op2 copy );
-        void internal_copy( const concurrent_vector_base_v3& src, size_type element_size, internal_array_op2 copy );
-        void internal_assign( const concurrent_vector_base_v3& src, size_type element_size,
+        size_type __TBB_EXPORTED_METHOD internal_grow_by( size_type delta, size_type element_size, internal_array_op2 init, const void *src );
+        void* __TBB_EXPORTED_METHOD internal_push_back( size_type element_size, size_type& index );
+        segment_index_t __TBB_EXPORTED_METHOD internal_clear( internal_array_op1 destroy );
+        void* __TBB_EXPORTED_METHOD internal_compact( size_type element_size, void *table, internal_array_op1 destroy, internal_array_op2 copy );
+        void __TBB_EXPORTED_METHOD internal_copy( const concurrent_vector_base_v3& src, size_type element_size, internal_array_op2 copy );
+        void __TBB_EXPORTED_METHOD internal_assign( const concurrent_vector_base_v3& src, size_type element_size,
                               internal_array_op1 destroy, internal_array_op2 assign, internal_array_op2 copy );
-        void internal_throw_exception(size_type) const;
-        void internal_swap(concurrent_vector_base_v3& v);
+        void __TBB_EXPORTED_METHOD internal_throw_exception(size_type) const;
+        void __TBB_EXPORTED_METHOD internal_swap(concurrent_vector_base_v3& v);
 
+        void __TBB_EXPORTED_METHOD internal_resize( size_type n, size_type element_size, size_type max_size, const void *src,
+                                                    internal_array_op1 destroy, internal_array_op2 init );
+        size_type __TBB_EXPORTED_METHOD internal_grow_to_at_least_with_result( size_type new_size, size_type element_size, internal_array_op2 init, const void *src );
+
+        //! Deprecated entry point for backwards compatibility to TBB 2.1.
+        void __TBB_EXPORTED_METHOD internal_grow_to_at_least( size_type new_size, size_type element_size, internal_array_op2 init, const void *src );
 private:
         //! Private functionality
         class helper;
@@ -167,16 +183,11 @@ private:
     
     typedef concurrent_vector_base_v3 concurrent_vector_base;
 
-    //TODO[?]: deal with _Range_checked_iterator_tag of MSVC
     //! Meets requirements of a forward iterator for STL and a Value for a blocked_range.*/
     /** Value is either the T or const T type of the container.
         @ingroup containers */
     template<typename Container, typename Value>
     class vector_iterator 
-#if defined(_WIN64) && defined(_MSC_VER) 
-        // Ensure that Microsoft's internal template function _Val_type works correctly.
-        : public std::iterator<std::random_access_iterator_tag,Value>
-#endif /* defined(_WIN64) && defined(_MSC_VER) */
     {
         //! concurrent_vector over which we are iterating.
         Container* my_vector;
@@ -210,10 +221,10 @@ private:
 public: // workaround for MSVC
 #endif 
 
-        vector_iterator( const Container& vector, size_t index ) : 
+        vector_iterator( const Container& vector, size_t index, void *ptr = 0 ) : 
             my_vector(const_cast<Container*>(&vector)), 
             my_index(index), 
-            my_item(NULL)
+            my_item(static_cast<Value*>(ptr))
         {}
 
     public:
@@ -229,7 +240,7 @@ public: // workaround for MSVC
         vector_iterator operator+( ptrdiff_t offset ) const {
             return vector_iterator( *my_vector, my_index+offset );
         }
-        vector_iterator operator+=( ptrdiff_t offset ) {
+        vector_iterator &operator+=( ptrdiff_t offset ) {
             my_index+=offset;
             my_item = NULL;
             return *this;
@@ -237,7 +248,7 @@ public: // workaround for MSVC
         vector_iterator operator-( ptrdiff_t offset ) const {
             return vector_iterator( *my_vector, my_index-offset );
         }
-        vector_iterator operator-=( ptrdiff_t offset ) {
+        vector_iterator &operator-=( ptrdiff_t offset ) {
             my_index-=offset;
             my_item = NULL;
             return *this;
@@ -316,7 +327,7 @@ public: // workaround for MSVC
 
     template<typename Container, typename T, typename U>
     bool operator==( const vector_iterator<Container,T>& i, const vector_iterator<Container,U>& j ) {
-        return i.my_index==j.my_index;
+        return i.my_index==j.my_index && i.my_vector == j.my_vector;
     }
 
     template<typename Container, typename T, typename U>
@@ -399,6 +410,13 @@ public: // workaround for MSVC
     may increase element access time. Internal layout can be optimized by method compact() that
     merges several smaller arrays into one solid.
 
+@par Changes since TBB 2.1
+    - Fixed guarantees of concurrent_vector::size() and grow_to_at_least() methods to assure elements are allocated.
+    - Methods end()/rbegin()/back() are partly thread-safe since they use size() to get the end of vector
+    - Added resize() methods (not thread-safe)
+    - Added cbegin/cend/crbegin/crend methods
+    - Changed return type of methods grow* and push_back to iterator
+
 @par Changes since TBB 2.0
     - Implemented exception-safety guaranties
     - Added template argument for allocator
@@ -415,12 +433,11 @@ public: // workaround for MSVC
     - Added compact() method to defragment first segments
     - Added swap() method
     - range() defaults on grainsize = 1 supporting auto grainsize algorithms. 
-    - clear() behavior changed to freeing segments memory 
 
     @ingroup containers */
 template<typename T, class A>
 class concurrent_vector: protected internal::allocator_base<T, A>,
-                         private internal::concurrent_vector_base_v3 {
+                         private internal::concurrent_vector_base {
 private:
     template<typename I>
     class generic_range_type: public blocked_range<I> {
@@ -487,7 +504,13 @@ public:
         : internal::allocator_base<T, A>(a)
     {
         vector_allocator_ptr = &internal_allocator;
-        internal_copy(vector, sizeof(T), &copy_array);
+        try {
+            internal_copy(vector, sizeof(T), &copy_array);
+        } catch(...) {
+            segment_t *table = my_segment;
+            internal_free_segments( reinterpret_cast<void**>(table), internal_clear(&destroy_array), my_first_block );
+            throw;
+        }
     }
 
     //! Copying constructor for vector with different allocator type
@@ -496,17 +519,26 @@ public:
         : internal::allocator_base<T, A>(a)
     {
         vector_allocator_ptr = &internal_allocator;
-        internal_copy(vector.internal_vector_base(), sizeof(T), &copy_array);
+        try {
+            internal_copy(vector.internal_vector_base(), sizeof(T), &copy_array);
+        } catch(...) {
+            segment_t *table = my_segment;
+            internal_free_segments( reinterpret_cast<void**>(table), internal_clear(&destroy_array), my_first_block );
+            throw;
+        }
     }
 
     //! Construction with initial size specified by argument n
     explicit concurrent_vector(size_type n)
     {
         vector_allocator_ptr = &internal_allocator;
-        if ( !n ) return;
-        internal_reserve(n, sizeof(T), max_size()); my_early_size = n;
-        __TBB_ASSERT( my_first_block == segment_index_of(n-1)+1, NULL );
-        initialize_array(static_cast<T*>(my_segment[0].array), NULL, n);
+        try {
+            internal_resize( n, sizeof(T), max_size(), NULL, &destroy_array, &initialize_array );
+        } catch(...) {
+            segment_t *table = my_segment;
+            internal_free_segments( reinterpret_cast<void**>(table), internal_clear(&destroy_array), my_first_block );
+            throw;
+        }
     }
 
     //! Construction with initial size specified by argument n, initialization by copying of t, and given allocator instance
@@ -514,7 +546,13 @@ public:
         : internal::allocator_base<T, A>(a)
     {
         vector_allocator_ptr = &internal_allocator;
-        internal_assign( n, t );
+        try {
+            internal_resize( n, sizeof(T), max_size(), static_cast<const void*>(&t), &destroy_array, &initialize_array_by );
+        } catch(...) {
+            segment_t *table = my_segment;
+            internal_free_segments( reinterpret_cast<void**>(table), internal_clear(&destroy_array), my_first_block );
+            throw;
+        }
     }
 
     //! Construction with copying iteration range and given allocator instance
@@ -523,13 +561,19 @@ public:
         : internal::allocator_base<T, A>(a)
     {
         vector_allocator_ptr = &internal_allocator;
-        internal_assign(first, last, static_cast<is_integer_tag<std::numeric_limits<I>::is_integer> *>(0) );
+        try {
+            internal_assign_range(first, last, static_cast<is_integer_tag<std::numeric_limits<I>::is_integer> *>(0) );
+        } catch(...) {
+            segment_t *table = my_segment;
+            internal_free_segments( reinterpret_cast<void**>(table), internal_clear(&destroy_array), my_first_block );
+            throw;
+        }
     }
 
     //! Assignment
     concurrent_vector& operator=( const concurrent_vector& vector ) {
         if( this != &vector )
-            concurrent_vector_base_v3::internal_assign(vector, sizeof(T), &destroy_array, &assign_array, &copy_array);
+            internal_assign(vector, sizeof(T), &destroy_array, &assign_array, &copy_array);
         return *this;
     }
 
@@ -537,7 +581,7 @@ public:
     template<class M>
     concurrent_vector& operator=( const concurrent_vector<T, M>& vector ) {
         if( static_cast<void*>( this ) != static_cast<const void*>( &vector ) )
-            concurrent_vector_base_v3::internal_assign(vector.internal_vector_base(),
+            internal_assign(vector.internal_vector_base(),
                 sizeof(T), &destroy_array, &assign_array, &copy_array);
         return *this;
     }
@@ -546,29 +590,70 @@ public:
     // Concurrent operations
     //------------------------------------------------------------------------
     //! Grow by "delta" elements.
+#if TBB_DEPRECATED
     /** Returns old size. */
     size_type grow_by( size_type delta ) {
         return delta ? internal_grow_by( delta, sizeof(T), &initialize_array, NULL ) : my_early_size;
     }
+#else
+    /** Returns iterator pointing to the first new element. */
+    iterator grow_by( size_type delta ) {
+        return iterator(*this, delta ? internal_grow_by( delta, sizeof(T), &initialize_array, NULL ) : my_early_size);
+    }
+#endif
 
     //! Grow by "delta" elements using copying constuctor.
+#if TBB_DEPRECATED
     /** Returns old size. */
     size_type grow_by( size_type delta, const_reference t ) {
         return delta ? internal_grow_by( delta, sizeof(T), &initialize_array_by, static_cast<const void*>(&t) ) : my_early_size;
     }
+#else
+    /** Returns iterator pointing to the first new element. */
+    iterator grow_by( size_type delta, const_reference t ) {
+        return iterator(*this, delta ? internal_grow_by( delta, sizeof(T), &initialize_array_by, static_cast<const void*>(&t) ) : my_early_size);
+    }
+#endif
 
-    //! Grow array until it has at least n elements.
+    //! Append minimal sequence of elements such that size()>=n.  
+#if TBB_DEPRECATED
+    /** The new elements are default constructed.  Blocks until all elements in range [0..n) are allocated.
+        May return while other elements are being constructed by other threads. */
     void grow_to_at_least( size_type n ) {
-        if( my_early_size<n )
-            internal_grow_to_at_least( n, sizeof(T), &initialize_array, NULL );
+        if( n ) internal_grow_to_at_least_with_result( n, sizeof(T), &initialize_array, NULL );
     };
+#else
+    /** The new elements are default constructed.  Blocks until all elements in range [0..n) are allocated.
+        May return while other elements are being constructed by other threads.
+        Returns iterator that points to beginning of appended sequence.
+        If no elements were appended, returns iterator pointing to nth element. */
+    iterator grow_to_at_least( size_type n ) {
+        size_type m=0;
+        if( n ) {
+            m = internal_grow_to_at_least_with_result( n, sizeof(T), &initialize_array, NULL );
+            if( m>n ) m=n;
+        }
+        return iterator(*this, m);
+    };
+#endif
 
     //! Push item 
-    size_type push_back( const_reference item ) {
+#if TBB_DEPRECATED
+    size_type push_back( const_reference item )
+#else
+    /** Returns iterator pointing to the new element. */
+    iterator push_back( const_reference item )
+#endif
+    {
         size_type k;
-        internal_loop_guide loop(1, internal_push_back(sizeof(T),k));
+        void *ptr = internal_push_back(sizeof(T),k);
+        internal_loop_guide loop(1, ptr);
         loop.init(&item);
+#if TBB_DEPRECATED
         return k;
+#else
+        return iterator(*this, k, ptr);
+#endif
     }
 
     //! Get reference to element at given index.
@@ -583,12 +668,12 @@ public:
         return internal_subscript(index);
     }
 
-    //! Get reference to element at given index.
+    //! Get reference to element at given index. Throws exceptions on errors.
     reference at( size_type index ) {
         return internal_subscript_with_exceptions(index);
     }
 
-    //! Get const reference to element at given index.
+    //! Get const reference to element at given index. Throws exceptions on errors.
     const_reference at( size_type index ) const {
         return internal_subscript_with_exceptions(index);
     }
@@ -605,13 +690,16 @@ public:
     //------------------------------------------------------------------------
     // Capacity
     //------------------------------------------------------------------------
-    //! Return size of vector.
-    size_type size() const {return my_early_size;}
+    //! Return size of vector. It may include elements under construction
+    size_type size() const {
+        size_type sz = my_early_size, cp = internal_capacity();
+        return cp < sz ? cp : sz;
+    }
 
-    //! Return size of vector.
+    //! Return true if vector is not empty or has elements under construction at least.
     bool empty() const {return !my_early_size;}
 
-    //! Maximum size to which array can grow without allocating more memory.
+    //! Maximum size to which array can grow without allocating more memory. Concurrent allocations are not included in the value.
     size_type capacity() const {return internal_capacity();}
 
     //! Allocate enough space to grow to size n without having to allocate more memory later.
@@ -622,8 +710,23 @@ public:
             internal_reserve(n, sizeof(T), max_size());
     }
 
-    //! Optimize memory usage and fragmentation. Returns true if optimization occurred.
-    void compact();
+    //! Resize the vector. Not thread-safe.
+    void resize( size_type n ) {
+        internal_resize( n, sizeof(T), max_size(), NULL, &destroy_array, &initialize_array );
+    }
+    
+    //! Resize the vector, copy t for new elements. Not thread-safe.
+    void resize( size_type n, const_reference t ) {
+        internal_resize( n, sizeof(T), max_size(), static_cast<const void*>(&t), &destroy_array, &initialize_array_by );
+    }
+   
+#if TBB_DEPRECATED 
+    //! An alias for shrink_to_fit()
+    void compact() {shrink_to_fit();}
+#endif /* TBB_DEPRECATED */
+
+    //! Optimize memory usage and fragmentation.
+    void shrink_to_fit();
 
     //! Upper bound on argument to reserve.
     size_type max_size() const {return (~size_type(0))/sizeof(T);}
@@ -640,6 +743,10 @@ public:
     const_iterator begin() const {return const_iterator(*this,0);}
     //! end const iterator
     const_iterator end() const {return const_iterator(*this,size());}
+    //! start const iterator
+    const_iterator cbegin() const {return const_iterator(*this,0);}
+    //! end const iterator
+    const_iterator cend() const {return const_iterator(*this,size());}
     //! reverse start iterator
     reverse_iterator rbegin() {return reverse_iterator(end());}
     //! reverse end iterator
@@ -648,6 +755,10 @@ public:
     const_reverse_iterator rbegin() const {return const_reverse_iterator(end());}
     //! reverse end const iterator
     const_reverse_iterator rend() const {return const_reverse_iterator(begin());}
+    //! reverse start const iterator
+    const_reverse_iterator crbegin() const {return const_reverse_iterator(end());}
+    //! reverse end const iterator
+    const_reverse_iterator crend() const {return const_reverse_iterator(begin());}
     //! the first item
     reference front() {
         __TBB_ASSERT( size()>0, NULL);
@@ -661,23 +772,26 @@ public:
     //! the last item
     reference back() {
         __TBB_ASSERT( size()>0, NULL);
-        return internal_subscript( my_early_size-1 );
+        return internal_subscript( size()-1 );
     }
     //! the last item const
     const_reference back() const {
         __TBB_ASSERT( size()>0, NULL);
-        return internal_subscript( my_early_size-1 );
+        return internal_subscript( size()-1 );
     }
     //! return allocator object
     allocator_type get_allocator() const { return this->my_allocator; }
 
     //! assign n items by copying t item
-    void assign(size_type n, const_reference t) { clear(); internal_assign( n, t ); }
+    void assign(size_type n, const_reference t) {
+        clear();
+        internal_resize( n, sizeof(T), max_size(), static_cast<const void*>(&t), &destroy_array, &initialize_array_by );
+    }
 
     //! assign range [first, last)
     template<class I>
     void assign(I first, I last) {
-        clear(); internal_assign( first, last, static_cast<is_integer_tag<std::numeric_limits<I>::is_integer> *>(0) );
+        clear(); internal_assign_range( first, last, static_cast<is_integer_tag<std::numeric_limits<I>::is_integer> *>(0) );
     }
 
     //! swap two instances
@@ -688,16 +802,16 @@ public:
         }
     }
 
-    //! Clear container. Not thread safe
+    //! Clear container while keeping memory allocated.
+    /** To free up the memory, use in conjunction with method compact(). Not thread safe **/
     void clear() {
-        segment_t *table = my_segment;
-        internal_free_segments( reinterpret_cast<void**>(table), internal_clear(&destroy_array), my_first_block );
-        my_first_block = 0; // here is not default_initial_segments
+        internal_clear(&destroy_array);
     }
 
     //! Clear and destroy vector.
     ~concurrent_vector() {
-        clear();
+        segment_t *table = my_segment;
+        internal_free_segments( reinterpret_cast<void**>(table), internal_clear(&destroy_array), my_first_block );
         // base class destructor call should be then
     }
 
@@ -717,19 +831,21 @@ private:
     T& internal_subscript_with_exceptions( size_type index ) const;
 
     //! assign n items by copying t
-    void internal_assign(size_type n, const_reference t);
+    void internal_assign_n(size_type n, const_pointer p) {
+        internal_resize( n, sizeof(T), max_size(), static_cast<const void*>(p), &destroy_array, p? &initialize_array_by : &initialize_array );
+    }
 
     //! helper class
     template<bool B> class is_integer_tag;
 
     //! assign integer items by copying when arguments are treated as iterators. See C++ Standard 2003 23.1.1p9
     template<class I>
-    void internal_assign(I first, I last, is_integer_tag<true> *) {
-        internal_assign(static_cast<size_type>(first), static_cast<T>(last));
+    void internal_assign_range(I first, I last, is_integer_tag<true> *) {
+        internal_assign_n(static_cast<size_type>(first), &static_cast<T&>(last));
     }
     //! inline proxy assign by iterators
     template<class I>
-    void internal_assign(I first, I last, is_integer_tag<false> *) {
+    void internal_assign_range(I first, I last, is_integer_tag<false> *) {
         internal_assign_iterators(first, last);
     }
     //! assign by iterators
@@ -737,22 +853,22 @@ private:
     void internal_assign_iterators(I first, I last);
 
     //! Construct n instances of T, starting at "begin".
-    static void initialize_array( void* begin, const void*, size_type n );
+    static void __TBB_EXPORTED_FUNC initialize_array( void* begin, const void*, size_type n );
 
     //! Construct n instances of T, starting at "begin".
-    static void initialize_array_by( void* begin, const void* src, size_type n );
+    static void __TBB_EXPORTED_FUNC initialize_array_by( void* begin, const void* src, size_type n );
 
     //! Construct n instances of T, starting at "begin".
-    static void copy_array( void* dst, const void* src, size_type n );
+    static void __TBB_EXPORTED_FUNC copy_array( void* dst, const void* src, size_type n );
 
     //! Assign n instances of T, starting at "begin".
-    static void assign_array( void* dst, const void* src, size_type n );
+    static void __TBB_EXPORTED_FUNC assign_array( void* dst, const void* src, size_type n );
 
     //! Destroy n instances of T, starting at "begin".
-    static void destroy_array( void* begin, size_type n );
+    static void __TBB_EXPORTED_FUNC destroy_array( void* begin, size_type n );
 
     //! Exception-aware helper class for filling a segment by exception-danger operators of user class
-    class internal_loop_guide {
+    class internal_loop_guide : internal::no_copy {
     public:
         const pointer array;
         const size_type n;
@@ -772,7 +888,7 @@ private:
 };
 
 template<typename T, class A>
-void concurrent_vector<T, A>::compact() {
+void concurrent_vector<T, A>::shrink_to_fit() {
     internal_segments_table old;
     try {
         if( internal_compact( sizeof(T), &old, &destroy_array, &copy_array ) )
@@ -791,11 +907,11 @@ void concurrent_vector<T, A>::internal_free_segments(void *table[], segment_inde
         --k;
         T* array = static_cast<T*>(table[k]);
         table[k] = NULL;
-        if( array > __TBB_BAD_ALLOC ) // check for correct segment pointer
+        if( array > internal::vector_allocation_error_flag ) // check for correct segment pointer
             this->my_allocator.deallocate( array, segment_size(k) );
     }
     T* array = static_cast<T*>(table[0]);
-    if( array > __TBB_BAD_ALLOC ) {
+    if( array > internal::vector_allocation_error_flag ) {
         __TBB_ASSERT( first_block > 0, NULL );
         while(k > 0) table[--k] = NULL;
         this->my_allocator.deallocate( array, segment_size(first_block) );
@@ -804,43 +920,53 @@ void concurrent_vector<T, A>::internal_free_segments(void *table[], segment_inde
 
 template<typename T, class A>
 T& concurrent_vector<T, A>::internal_subscript( size_type index ) const {
-    __TBB_ASSERT( index<size(), "index out of bounds" );
+    __TBB_ASSERT( index < my_early_size, "index out of bounds" );
     size_type j = index;
     segment_index_t k = segment_base_index_of( j );
+    __TBB_ASSERT( my_segment != (segment_t*)my_storage || k < pointers_per_short_table, "index is being allocated" );
     // no need in __TBB_load_with_acquire since thread works in own space or gets 
-    return static_cast<T*>(my_segment[k].array)[j];
+#if TBB_USE_THREADING_TOOLS
+    T* array = static_cast<T*>( tbb::internal::itt_load_pointer_v3(&my_segment[k].array));
+#else
+    T* array = static_cast<T*>(my_segment[k].array);
+#endif /* TBB_USE_THREADING_TOOLS */
+    __TBB_ASSERT( array != internal::vector_allocation_error_flag, "the instance is broken by bad allocation. Use at() instead" );
+    __TBB_ASSERT( array, "index is being allocated" );
+    return array[j];
 }
 
 template<typename T, class A>
 T& concurrent_vector<T, A>::internal_subscript_with_exceptions( size_type index ) const {
-    if( index >= size() )
+    if( index >= my_early_size )
         internal_throw_exception(0); // throw std::out_of_range
     size_type j = index;
     segment_index_t k = segment_base_index_of( j );
     if( my_segment == (segment_t*)my_storage && k >= pointers_per_short_table )
-        internal_throw_exception(1); // throw std::out_of_range
+        internal_throw_exception(1); // throw std::range_error
     void *array = my_segment[k].array; // no need in __TBB_load_with_acquire
-    if( array <= __TBB_BAD_ALLOC ) // check for correct segment pointer
+    if( array <= internal::vector_allocation_error_flag ) // check for correct segment pointer
         internal_throw_exception(2); // throw std::range_error
     return static_cast<T*>(array)[j];
 }
 
-template<typename T, class A>
-void concurrent_vector<T, A>::internal_assign(size_type n, const_reference t)
-{
-    if( !n ) return;
-    internal_reserve(n, sizeof(T), max_size()); my_early_size = n;
-    __TBB_ASSERT( my_first_block == segment_index_of(n-1)+1, NULL );
-    initialize_array_by(static_cast<T*>(my_segment[0].array), static_cast<const void*>(&t), n);
-}
-
 template<typename T, class A> template<class I>
 void concurrent_vector<T, A>::internal_assign_iterators(I first, I last) {
+    __TBB_ASSERT(my_early_size == 0, NULL);
     size_type n = std::distance(first, last);
     if( !n ) return;
-    internal_reserve(n, sizeof(T), max_size()); my_early_size = n;
-    __TBB_ASSERT( my_first_block == segment_index_of(n-1)+1, NULL );
-    internal_loop_guide loop(n, my_segment[0].array); loop.iterate(first);
+    internal_reserve(n, sizeof(T), max_size());
+    my_early_size = n;
+    segment_index_t k = 0;
+    size_type sz = segment_size( my_first_block );
+    while( sz < n ) {
+        internal_loop_guide loop(sz, my_segment[k].array);
+        loop.iterate(first);
+        n -= sz;
+        if( !k ) k = my_first_block;
+        else { ++k; sz <<= 1; }
+    }
+    internal_loop_guide loop(n, my_segment[k].array);
+    loop.iterate(first);
 }
 
 template<typename T, class A>
@@ -863,17 +989,24 @@ void concurrent_vector<T, A>::assign_array( void* dst, const void* src, size_typ
     internal_loop_guide loop(n, dst); loop.assign(src);
 }
 
+#if defined(_MSC_VER) && !defined(__INTEL_COMPILER) 
+    // Workaround for overzealous compiler warning
+    #pragma warning (push)
+    #pragma warning (disable: 4189)
+#endif
 template<typename T, class A>
 void concurrent_vector<T, A>::destroy_array( void* begin, size_type n ) {
     T* array = static_cast<T*>(begin);
     for( size_type j=n; j>0; --j )
         array[j-1].~T(); // destructors are supposed to not throw any exceptions
 }
+#if defined(_MSC_VER) && !defined(__INTEL_COMPILER) 
+    #pragma warning (pop)
+#endif // warning 4189 is back 
 
 // concurrent_vector's template functions
 template<typename T, class A1, class A2>
 inline bool operator==(const concurrent_vector<T, A1> &a, const concurrent_vector<T, A2> &b) {
-    //TODO[?]: deal with _Range_checked_iterator_tag of MSVC.
     // Simply:    return a.size() == b.size() && std::equal(a.begin(), a.end(), b.begin());
     if(a.size() != b.size()) return false;
     typename concurrent_vector<T, A1>::const_iterator i(a.begin());
@@ -909,9 +1042,8 @@ inline void swap(concurrent_vector<T, A> &a, concurrent_vector<T, A> &b)
 
 } // namespace tbb
 
-#if defined(_MSC_VER) && defined(_Wp64)
-    // Workaround for overzealous compiler warnings in /Wp64 mode
+#if defined(_MSC_VER) && !defined(__INTEL_COMPILER) && defined(_Wp64)
     #pragma warning (pop)
-#endif /* _MSC_VER && _Wp64 */
+#endif // warning 4267 is back
 
 #endif /* __TBB_concurrent_vector_H */
