@@ -323,10 +323,17 @@ namespace internal {
     See also requirements on \ref range_req "Range" and \ref parallel_scan_body_req "parallel_scan Body". **/
 //@{
 
+//! Parallel prefix with default partitioner
+/** @ingroup algorithms **/
+template<typename Range, typename Body>
+void parallel_scan( const Range& range, Body& body ) {
+    internal::start_scan<Range,Body,__TBB_DEFAULT_PARTITIONER>::run(range,body,__TBB_DEFAULT_PARTITIONER());
+}
+
 //! Parallel prefix with simple_partitioner
 /** @ingroup algorithms **/
 template<typename Range, typename Body>
-void parallel_scan( const Range& range, Body& body, const simple_partitioner& partitioner=simple_partitioner() ) {
+void parallel_scan( const Range& range, Body& body, const simple_partitioner& partitioner ) {
     internal::start_scan<Range,Body,simple_partitioner>::run(range,body,partitioner);
 }
 
