@@ -32,6 +32,10 @@
 #include "tbb_stddef.h"
 #include <stdexcept>
 
+#if __TBB_EXCEPTIONS && !defined(__EXCEPTIONS) && !defined(_CPPUNWIND) && !defined(__SUNPRO_CC)
+#error The current compilation environment does not support exception handling. Please set __TBB_EXCEPTIONS to 0 in tbb_config.h
+#endif
+
 namespace tbb {
 
 //! Exception for concurrent containers
@@ -41,6 +45,10 @@ public:
     virtual ~bad_last_alloc() throw() {}
 };
 
+namespace internal {
+void __TBB_EXPORTED_FUNC throw_bad_last_alloc_exception_v4() ;
+} // namespace internal
+
 } // namespace tbb
 
 #if __TBB_EXCEPTIONS
@@ -72,7 +80,13 @@ namespace tbb {
     
     TBB provides two implementations of this interface: tbb::captured_exception and 
     template class tbb::movable_exception. See their declarations for more info. **/
-class tbb_exception : public std::exception {
+class tbb_exception : public std::exception
+{
+    /** No operator new is provided because the TBB usage model assumes dynamic 
+        creation of the TBB exception objects only by means of applying move()
+        operation on an exception thrown out of TBB scheduler. **/
+    void* operator new ( size_t );
+
 public:
     //! Creates and returns pointer to the deep copy of this exception object. 
     /** Move semantics is allowed. **/
@@ -95,6 +109,16 @@ public:
 
     //! Returns the result of originally intercepted exception's what() method.
     virtual const char* what() const throw() = 0;
+
+    /** Operator delete is provided only to allow using existing smart pointers
+        with TBB exception objects obtained as the result of applying move()
+        operation on an exception thrown out of TBB scheduler. 
+        
+        When overriding method move() make sure to override operator delete as well
+        if memory is allocated not by TBB's scalable allocator. **/
+    void operator delete ( void* p ) {
+        internal::deallocate_via_handler_v3(p);
+    }
 };
 
 //! This class is used by TBB to propagate information about unhandled exceptions into the root thread.
@@ -106,7 +130,7 @@ class captured_exception : public tbb_exception
 {
 public:
     captured_exception ( const captured_exception& src )
-        : my_dynamic(false)
+        : tbb_exception(src), my_dynamic(false)
     {
         set(src.my_exception_name, src.my_exception_info);
     }
@@ -130,10 +154,10 @@ public:
     }
 
     /*override*/ 
-    captured_exception* move () throw();
+    captured_exception* __TBB_EXPORTED_METHOD move () throw();
 
     /*override*/ 
-    void destroy () throw();
+    void __TBB_EXPORTED_METHOD destroy () throw();
 
     /*override*/ 
     void throw_self () { throw *this; }
@@ -144,16 +168,16 @@ public:
     /*override*/ 
     const char* __TBB_EXPORTED_METHOD what() const throw();
 
+    void __TBB_EXPORTED_METHOD set ( const char* name, const char* info ) throw();
+    void __TBB_EXPORTED_METHOD clear () throw();
+
 private:
     //! Used only by method clone().  
     captured_exception() {}
 
-    //! Functionally equivalent to {captured_exception e(name,info); return c.clone();}
+    //! Functionally equivalent to {captured_exception e(name,info); return e.clone();}
     static captured_exception* allocate ( const char* name, const char* info );
 
-    void set ( const char* name, const char* info ) throw();
-    void clear () throw();
-
     bool my_dynamic;
     const char* my_exception_name;
     const char* my_exception_info;
@@ -177,7 +201,8 @@ public:
     {}
 
     movable_exception ( const movable_exception& src ) throw () 
-        : my_exception_data(src.my_exception_data)
+        : tbb_exception(src)
+        , my_exception_data(src.my_exception_data)
         , my_dynamic(false)
         , my_exception_name(src.my_exception_name)
     {}
@@ -204,7 +229,7 @@ public:
     movable_exception* move () throw() {
         void* e = internal::allocate_via_handler_v3(sizeof(movable_exception));
         if ( e ) {
-            new (e) movable_exception(*this);
+            ::new (e) movable_exception(*this);
             ((movable_exception*)e)->my_dynamic = true;
         }
         return (movable_exception*)e;
@@ -235,6 +260,36 @@ private:
     const char* my_exception_name;
 };
 
+#if !TBB_USE_CAPTURED_EXCEPTION
+namespace internal {
+
+//! Exception container that preserves the exact copy of the original exception
+/** This class can be used only when the appropriate runtime support (mandated 
+    by C++0x) is present **/
+class tbb_exception_ptr {
+    std::exception_ptr  my_ptr;
+
+public:
+    static tbb_exception_ptr* allocate ();
+    static tbb_exception_ptr* allocate ( const tbb_exception& tag );
+    //! This overload uses move semantics (i.e. it empties src)
+    static tbb_exception_ptr* allocate ( captured_exception& src );
+    
+    //! Destroys this objects
+    /** Note that objects of this type can be created only by the allocate() method. **/
+    void destroy () throw();
+
+    //! Throws the contained exception .
+    void throw_self () { std::rethrow_exception(my_ptr); }
+
+private:
+    tbb_exception_ptr ( const std::exception_ptr& src ) : my_ptr(src) {}
+    tbb_exception_ptr ( const captured_exception& src ) : my_ptr(std::copy_exception(src)) {}
+}; // class tbb::internal::tbb_exception_ptr
+
+} // namespace internal
+#endif /* !TBB_USE_CAPTURED_EXCEPTION */
+
 } // namespace tbb
 
 #endif /* __TBB_EXCEPTIONS */
