@@ -26,13 +26,11 @@
     the GNU General Public License.
 */
 
-#define INSTANTIATE_ITT_NOTIFY 1
 #include "TypeDefinitions.h" // Customize.h and proxy.h get included
 
-#define ITT_DoOneTimeInitialization __TBB_malloc_ITT_DoOneTimeInitialization_stub
 #include "tbb/itt_notify.cpp"
 
-#if MALLOC_LD_PRELOAD
+#if MALLOC_CHECK_RECURSION
 
 #include <pthread.h>
 #include <stdio.h>
@@ -42,42 +40,54 @@
 #include <errno.h>
 #endif
 
+#if MALLOC_LD_PRELOAD
+
 extern "C" {
 
 void   safer_scalable_free( void*, void (*)(void*) );
-void * safer_scalable_realloc( void*, size_t, void* (*)(void*,size_t) );
+void * safer_scalable_realloc( void*, size_t, void* );
 
 bool __TBB_internal_find_original_malloc(int num, const char *names[], void *table[])  __attribute__ ((weak));
 
 }
 
-#define malloc_proxy __TBB_malloc_proxy
+#endif /* MALLOC_LD_PRELOAD */
+#endif /* MALLOC_CHECK_RECURSION */
+
+namespace rml {
+namespace internal {
+
+#if MALLOC_CHECK_RECURSION
 
 void* (*original_malloc_ptr)(size_t) = 0;
 void  (*original_free_ptr)(void*) = 0;
 static void* (*original_calloc_ptr)(size_t,size_t) = 0;
 static void* (*original_realloc_ptr)(void*,size_t) = 0;
 
-#endif /* MALLOC_LD_PRELOAD */
-
-namespace tbb {
-namespace internal {
+#endif /* MALLOC_CHECK_RECURSION */
 
+#if __TBB_NEW_ITT_NOTIFY
+extern "C" 
+#endif
 void ITT_DoOneTimeInitialization() {} // required for itt_notify.cpp to work
 
 #if DO_ITT_NOTIFY
 /** Caller is responsible for ensuring this routine is called exactly once. */
 void MallocInitializeITT() {
+#if __TBB_NEW_ITT_NOTIFY
+    tbb::internal::__TBB_load_ittnotify();
+#else
     bool success = false;
     // Check if we are running under control of VTune.
     if( GetBoolEnvironmentVariable("KMP_FOR_TCHECK") || GetBoolEnvironmentVariable("KMP_FOR_TPROFILE") ) {
         // Yes, we are under control of VTune.  Check for libittnotify library.
-        success = FillDynamicLinks( LIBITTNOTIFY_NAME, ITT_HandlerTable, 5 );
+        success = dynamic_link( LIBITTNOTIFY_NAME, ITT_HandlerTable, 5 );
     }
     if (!success){
         for (int i = 0; i < 5; i++)
             *ITT_HandlerTable[i].handler = NULL;
     }
+#endif /* !__TBB_NEW_ITT_NOTIFY */
 }
 #endif /* DO_ITT_NOTIFY */
 
@@ -102,6 +112,7 @@ void init_tbbmalloc() {
         }
     }
 #endif /* MALLOC_LD_PRELOAD */
+
 #if DO_ITT_NOTIFY
     MallocInitializeITT();
 #endif
@@ -117,12 +128,50 @@ struct RegisterProcessShutdownNotification {
 static RegisterProcessShutdownNotification reg;
 #endif
 
+#if MALLOC_CHECK_RECURSION
+
+bool  original_malloc_found;
+
+#if MALLOC_LD_PRELOAD
+
+extern "C" {
+
+void * __TBB_internal_malloc(size_t size)
+{
+    return scalable_malloc(size);
+}
+
+void * __TBB_internal_calloc(size_t num, size_t size)
+{
+    return scalable_calloc(num, size);
+}
+
+int __TBB_internal_posix_memalign(void **memptr, size_t alignment, size_t size)
+{
+    return scalable_posix_memalign(memptr, alignment, size);
+}
+
+void* __TBB_internal_realloc(void* ptr, size_t sz)
+{
+    return safer_scalable_realloc(ptr, sz, (void*&)original_realloc_ptr);
+}
+
+void __TBB_internal_free(void *object)
+{
+    safer_scalable_free(object, original_free_ptr);
+}
+
+} /* extern "C" */
+
+#endif /* MALLOC_LD_PRELOAD */
+#endif /* MALLOC_CHECK_RECURSION */
+
 } } // namespaces
 
 #ifdef _WIN32
 #include <windows.h>
 
-BOOL WINAPI DllMain( HINSTANCE hInst, DWORD callReason, LPVOID )
+extern "C" BOOL WINAPI DllMain( HINSTANCE hInst, DWORD callReason, LPVOID )
 {
 
     if (callReason==DLL_THREAD_DETACH)
@@ -138,82 +187,3 @@ BOOL WINAPI DllMain( HINSTANCE hInst, DWORD callReason, LPVOID )
 
 #endif //_WIN32
 
-#if MALLOC_LD_PRELOAD
-
-bool  original_malloc_found;
-
-pthread_t   recursive_malloc_call_thread;
-int         recursive_malloc_call_flag;
-
-void lockRecursiveMallocFlag()
-{
-    recursive_malloc_call_thread = pthread_self();
-    recursive_malloc_call_flag = 1;
-}
-void unlockRecursiveMallocFlag()
-{
-    recursive_malloc_call_flag = 0;
-}
-static inline bool underRecursiveMallocFlag()
-{
-    return recursive_malloc_call_flag
-        && pthread_equal( recursive_malloc_call_thread, pthread_self());
-}
-
-extern "C" {
-
-void * __TBB_internal_malloc(size_t size)
-{
-    if ( underRecursiveMallocFlag() ) {
-        if ( original_malloc_found ){
-            return original_malloc_ptr(size);
-        }else{
-            return NULL;
-        }
-    }else{
-        return (void*)scalable_malloc(size);
-    }
-}
-
-void * __TBB_internal_calloc(size_t num, size_t size)
-{
-    if ( underRecursiveMallocFlag() ) {
-        if ( original_malloc_found ){
-#if __sun
-            /* There seem to be run time problems on Solaris if original_calloc_ptr is used. */
-            size_t arraySize = num * size;
-            void *result = original_malloc_ptr(arraySize);
-            if (result)
-                memset(result, 0, arraySize);
-            return result;
-#else
-            return original_calloc_ptr(num, size);
-#endif
-        }else{
-            return NULL;
-        }
-    }else{
-        return scalable_calloc(num, size);
-    }
-}
-
-int __TBB_internal_posix_memalign(void **memptr, size_t alignment, size_t size)
-{
-    MALLOC_ASSERT( !underRecursiveMallocFlag(), 
-                   "posix_memalign not expected to create a cyclic dependency" );
-    return scalable_posix_memalign(memptr, alignment, size);
-}
-
-void* __TBB_internal_realloc(void* ptr, size_t sz)
-{
-    return safer_scalable_realloc(ptr, sz, original_realloc_ptr);
-}
-
-void __TBB_internal_free(void *object)
-{
-    safer_scalable_free(object, original_free_ptr);
-}
-
-} /* extern "C" */
-
-#endif /* MALLOC_LD_PRELOAD */
