@@ -31,17 +31,27 @@
 // Every test is presumed to have a command line of the form "foo [-v] [nthread]"
 // The default for nthread is 2.
 
+#ifndef tbb_tests_harness_H
+#define tbb_tests_harness_H
+
+#define __TBB_LAMBDAS_PRESENT  ( _MSC_VER >= 1600 && !__INTEL_COMPILER || __INTEL_COMPILER > 1100 && _TBB_CPP0X )
+
 #if __SUNPRO_CC
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #else
-#include <cstdio>
 #include <cstdlib>
 #include <cstring>
 #endif
 #include <new>
 
+#if __LRB__
+#include "harness_lrb.h"
+#else
+#define __TBB_TEST_EXPORT
+#define REPORT_FATAL_ERROR REPORT
+#endif /* !__LRB__ */
+
 #if _WIN32||_WIN64
     #include <windows.h>
     #include <process.h>
@@ -53,23 +63,42 @@
 #include <errno.h>       /* for use in LinuxKernelVersion() */
 #endif
 
+#include "harness_report.h"
+
 #if !HARNESS_NO_ASSERT
 #include "harness_assert.h"
 
-static void ReportError( int line, const char* expression, const char * message, bool is_error ) {
-    if ( is_error ) {
-        printf("Line %d, assertion %s: %s\n", line, expression, message ? message : "failed" );
-#if TBB_EXIT_ON_ASSERT
-        exit(1);
+typedef void (*test_error_extra_t)(void);
+static test_error_extra_t ErrorExtraCall; 
+//! Set additional handler to process failed assertions
+void SetHarnessErrorProcessing( test_error_extra_t extra_call ) {
+    ErrorExtraCall = extra_call;
+    // TODO: add tbb::set_assertion_handler(ReportError);
+}
+//! Reports errors issued by failed assertions
+void ReportError( const char* filename, int line, const char* expression, const char * message ) {
+#if __TBB_ICL_11_1_CODE_GEN_BROKEN
+    printf("%s:%d, assertion %s: %s\n", filename, line, expression, message ? message : "failed" );
 #else
-        abort();
+    REPORT_FATAL_ERROR("%s:%d, assertion %s: %s\n", filename, line, expression, message ? message : "failed" );
+#endif
+    if( ErrorExtraCall )
+        (*ErrorExtraCall)();
+#if TBB_TERMINATE_ON_ASSERT
+    TerminateProcess(GetCurrentProcess(), 1);
+#elif TBB_EXIT_ON_ASSERT
+    exit(1);
+#else
+    abort();
 #endif /* TBB_EXIT_ON_ASSERT */
-    }
-    else
-        printf("Warning: at line %d, assertion %s: %s\n", line, expression, message ? message : "failed" );
+}
+//! Reports warnings issued by failed warning assertions
+void ReportWarning( const char* filename, int line, const char* expression, const char * message ) {
+    REPORT("Warning: %s:%d, assertion %s: %s\n", filename, line, expression, message ? message : "failed" );
 }
 #else
-#define ASSERT(x,y) ((void)0)
+#define ASSERT(p,msg) ((void)0)
+#define ASSERT_WARNING(p,msg) ((void)0)
 #endif /* HARNESS_NO_ASSERT */
 
 #if !HARNESS_NO_PARSE_COMMAND_LINE
@@ -106,21 +135,29 @@ static void ParseCommandLine( int argc, char* argv[] ) {
         else if( *endptr=='\0' ) 
             MaxThread = MinThread;
         if( *endptr!='\0' ) {
-            printf("garbled nthread range\n");
+            REPORT_FATAL_ERROR("garbled nthread range\n");
             exit(1);
         }    
         if( MinThread<0 ) {
-            printf("nthread must be nonnegative\n");
+            REPORT_FATAL_ERROR("nthread must be nonnegative\n");
             exit(1);
         }
         if( MaxThread<MinThread ) {
-            printf("nthread range is backwards\n");
+            REPORT_FATAL_ERROR("nthread range is backwards\n");
             exit(1);
         }
         ++i;
     }
+#if __TBB_STDARGS_BROKEN
+    if ( !argc )
+        argc = 1;
+    else {
+        while ( i < argc && argv[i][0] == 0 )
+            ++i;
+    }
+#endif /* __TBB_STDARGS_BROKEN */
     if( i!=argc ) {
-        printf("Usage: %s [-v] [nthread|minthread:maxthread]\n", argv[0] );
+        REPORT_FATAL_ERROR("Usage: %s [-v] [nthread|minthread:maxthread]\n", argv[0] );
         exit(1);
     }
 }
@@ -165,8 +202,26 @@ public:
     #pragma warning (push)
     #pragma warning (disable: 2193)
 #endif /* __ICC==1100 */
-        int status = pthread_create(&thread_id, NULL, thread_function, this);
+        // Some machines may have very large hard stack limit. When the test is 
+        // launched by make, the default stack size is set to the hard limit, and 
+        // calls to pthread_create fail with out-of-memory error. 
+        // Therefore we set the stack size explicitly (as for TBB worker threads).
+        const size_t MByte = 1<<20;
+#if __i386__||__i386
+        const size_t stack_size = 1*MByte;
+#elif __x86_64__
+        const size_t stack_size = 2*MByte;
+#else
+        const size_t stack_size = 4*MByte;
+#endif
+        pthread_attr_t attr_stack;
+        int status = pthread_attr_init(&attr_stack);
+        ASSERT(0==status, "NativeParallelFor: pthread_attr_init failed");
+        status = pthread_attr_setstacksize( &attr_stack, stack_size );
+        ASSERT(0==status, "NativeParallelFor: pthread_attr_setstacksize failed");
+        status = pthread_create(&thread_id, &attr_stack, thread_function, this);
         ASSERT(0==status, "NativeParallelFor: pthread_create failed");
+        pthread_attr_destroy(&attr_stack);
 #if __ICC==1100
     #pragma warning (pop)
 #endif
@@ -205,13 +260,21 @@ private:
 #endif
     {
         NativeParallelForTask& self = *static_cast<NativeParallelForTask*>(object);
+#if defined(__EXCEPTIONS) || defined(_CPPUNWIND) || defined(__SUNPRO_CC)
+        try {
+            (self.body)(self.index);
+        } catch(...) {
+            ASSERT( false, "uncaught exception" );
+        }
+#else
         (self.body)(self.index);
+#endif// exceptions are enabled
         return 0;
     }
 };
 
 //! Execute body(i) in parallel for i in the interval [0,n).
-/** Each iteartion is performed by a separate thread. */
+/** Each iteration is performed by a separate thread. */
 template<typename Index, typename Body>
 void NativeParallelFor( Index n, const Body& body ) {
     typedef NativeParallelForTask<Index,Body> task;
@@ -251,7 +314,7 @@ void zero_fill(void* array, size_t N) {
         In case operands cause signed/unsigned or size mismatch warnings it is caller's
         responsibility to do the appropriate cast before calling the function. **/
     template<typename T1, typename T2>
-    const T1& min ( const T1& val1, const T2& val2 ) {
+    T1 min ( const T1& val1, const T2& val2 ) {
         return val1 < val2 ? val1 : val2;
     }
 #endif /* !min */
@@ -262,7 +325,7 @@ void zero_fill(void* array, size_t N) {
         In case operands cause signed/unsigned or size mismatch warnings it is caller's
         responsibility to do the appropriate cast before calling the function. **/
     template<typename T1, typename T2>
-    const T1& max ( const T1& val1, const T2& val2 ) {
+    T1 max ( const T1& val1, const T2& val2 ) {
         return val1 < val2 ? val2 : val1;
     }
 #endif /* !max */
@@ -272,15 +335,63 @@ inline unsigned LinuxKernelVersion()
 {
     unsigned a, b, c;
     struct utsname utsnameBuf;
-
+    
     if (-1 == uname(&utsnameBuf)) {
-        printf("Can't call uname: errno %d\n", errno);
+        REPORT_FATAL_ERROR("Can't call uname: errno %d\n", errno);
         exit(1);
     }
     if (3 != sscanf(utsnameBuf.release, "%u.%u.%u", &a, &b, &c)) {
-        printf("Unable to parse OS release '%s'\n", utsnameBuf.release);
+        REPORT_FATAL_ERROR("Unable to parse OS release '%s'\n", utsnameBuf.release);
         exit(1);
     }
     return 1000000*a+1000*b+c;
 }
 #endif
+
+namespace Harness {
+
+#if !HARNESS_NO_ASSERT
+//! Base class that asserts that no operations are made with the object after its destruction.
+class NoAfterlife {
+protected:
+    enum state_t {
+        LIVE=0x56781234,
+        DEAD=0xDEADBEEF
+    } m_state;
+
+public:
+    NoAfterlife() : m_state(LIVE) {}
+    NoAfterlife( const NoAfterlife& src ) : m_state(LIVE) {
+        ASSERT( src.IsLive(), "Constructing from the dead source" );
+    }
+    ~NoAfterlife() {
+        ASSERT( IsLive(), "Repeated destructor call" );
+        m_state = DEAD;
+    }
+    const NoAfterlife& operator=( const NoAfterlife& src ) {
+        ASSERT( IsLive(), NULL );
+        ASSERT( src.IsLive(), NULL );
+        return *this;
+    }
+    void AssertLive() const {
+        ASSERT( IsLive(), "Already dead" );
+    }
+    bool IsLive() const {
+        return m_state == LIVE;
+    }
+}; // NoAfterlife
+#endif /* !HARNESS_NO_ASSERT */
+
+#if _WIN32 || _WIN64
+    void Sleep ( int ms ) { ::Sleep(ms); }
+#else /* !WIN */
+    void Sleep ( int ms ) {
+        timespec  requested = { ms / 1000, (ms % 1000)*1000000 };
+        timespec  remaining = {0};
+        nanosleep(&requested, &remaining);
+    }
+#endif /* !WIN */
+
+} // namespace Harness
+
+#endif /* tbb_tests_harness_H */
