@@ -26,6 +26,12 @@
     the GNU General Public License.
 */
 
+#include "tbb/tbb_config.h"
+
+#if __TBB_GCC_WARNING_SUPPRESSION_ENABLED
+#pragma GCC diagnostic ignored "-Wstrict-aliasing"
+#endif
+
 //! Wrapper around T where all members are private.
 /** Used to prove that aligned_space<T,N> never calls member of T. */
 template<typename T>
@@ -35,12 +41,15 @@ class Minimal {
     ~Minimal();
     void operator=( const Minimal& );
     T pad;
-public:
-    friend void AssignToCheckAlignment( Minimal& dst, const Minimal& src ) {
-        dst.pad = src.pad;
-    }  
+    template<typename U>
+    friend void AssignToCheckAlignment( Minimal<U>& dst, const Minimal<U>& src ) ;
 };
 
+template<typename T>
+void AssignToCheckAlignment( Minimal<T>& dst, const Minimal<T>& src ) {
+    dst.pad = src.pad;
+}
+
 #include "tbb/aligned_space.h"
 #include "harness_assert.h"
 
@@ -85,9 +94,12 @@ void TestAlignedSpace() {
         PrintSpaceWastingWarning( typeid(T).name() );
 }
 
-#include "harness_m128.h"
-#include <cstdio>         // Inclusion of <cstdio> deferred, to improve odds of detecting accidental dependences on it.
+#define HARNESS_NO_PARSE_COMMAND_LINE 1
+#include "harness.h"
 
+#include "harness_m128.h"
+
+__TBB_TEST_EXPORT
 int main() {
     TestAlignedSpace<char>();
     TestAlignedSpace<short>();
@@ -99,14 +111,11 @@ int main() {
 #if HAVE_m128
     TestAlignedSpace<__m128>();
 #endif /* HAVE_m128 */
-    std::printf("done\n");
+    REPORT("done\n");
     return 0;
 }
 
-#define HARNESS_NO_PARSE_COMMAND_LINE 1
-#include "harness.h"
-
 static void PrintSpaceWastingWarning( const char* type_name ) {
-    std::printf("Consider rewriting aligned_space<%s,N> to waste less space\n", type_name ); 
+    REPORT("Consider rewriting aligned_space<%s,N> to waste less space\n", type_name ); 
 }
 
