@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2008 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2009 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -31,6 +31,13 @@
 
 #include "tbb/atomic.h"
 #include "harness_assert.h"
+#include <string.h> // memcmp
+
+#if _MSC_VER && !defined(__INTEL_COMPILER)
+    // unary minus operator applied to unsigned type, result still unsigned
+    #pragma warning( push )
+    #pragma warning( disable: 4310 )
+#endif
 
 //! Structure that holds an atomic<T> and some guard bytes around it.
 template<typename T>
@@ -49,6 +56,10 @@ struct TestStruct {
     }
 };
 
+#if _MSC_VER && !defined(__INTEL_COMPILER)
+    #pragma warning( pop )
+#endif
+
 //! Test compare_and_swap template members of class atomic<T> for memory_semantics=M
 template<typename T,tbb::memory_semantics M>
 void TestCompareAndSwapAcquireRelease( T i, T j, T k ) {
@@ -107,6 +118,13 @@ void TestFetchAndStore( T i, T j ) {
     TestFetchAndStoreAcquireRelease<T,tbb::release>(i,j);
 }
 
+#if _MSC_VER && !defined(__INTEL_COMPILER)
+    // conversion from <bigger integer> to <smaller integer>, possible loss of data
+    // the warning seems a complete nonsense when issued for e.g. short+=short
+    #pragma warning( push )
+    #pragma warning( disable: 4244 )
+#endif
+
 //! Test fetch_and_add members of class atomic<T> for memory_semantics=M
 template<typename T,tbb::memory_semantics M>
 void TestFetchAndAddAcquireRelease( T i ) {
@@ -180,6 +198,16 @@ void TestFetchAndAdd( T i ) {
     TestFetchAndAddAcquireRelease<T,tbb::release>(i);
 }
 
+#if _MSC_VER && !defined(__INTEL_COMPILER)
+    #pragma warning( pop )
+#endif // warning 4244 is back
+
+//! A type with unknown size.
+class IncompleteType;
+
+void TestFetchAndAdd( IncompleteType* ) {
+    // There are no fetch-and-add operations on a IncompleteType*.
+}
 void TestFetchAndAdd( void* ) {
     // There are no fetch-and-add operations on a void*.
 }
@@ -192,7 +220,7 @@ template<typename T>
 void TestConst( T i ) { 
     // Try const 
     const TestStruct<T> x(i);
-    ASSERT( reinterpret_cast<const T&>(x.counter)==i, "write to atomic<T> broken?" );
+    ASSERT( memcmp( &i, &x.counter, sizeof(T) )==0, "write to atomic<T> broken?" );;
     ASSERT( x.counter==i, "read of atomic<T> broken?" );
 }
 
@@ -201,7 +229,6 @@ void TestOperations( T i, T j, T k ) {
     TestConst(i);
     TestCompareAndSwap(i,j,k);
     TestFetchAndStore(i,k);    // Pass i,k instead of i,j, because callee requires two distinct values.
-    TestFetchAndAdd(i);
 }
 
 template<typename T>
@@ -218,22 +245,21 @@ struct AlignmentChecker {
 #include "harness.h"
 
 #if _MSC_VER && !defined(__INTEL_COMPILER)
-#pragma warning( push )
-// unary minus operator applied to unsigned type, result still unsigned
-#pragma warning( disable: 4146 )
-#endif /* _MSC_VER && !defined(__INTEL_COMPILER) */
+    // unary minus operator applied to unsigned type, result still unsigned
+    #pragma warning( push )
+    #pragma warning( disable: 4146 )
+#endif
 
 /** T is an integral type. */
 template<typename T>
 void TestAtomicInteger( const char* name ) {
-    if( Verbose )
-        printf("testing atomic<%s>\n",name);
+    REMARK("testing atomic<%s>\n",name);
 #if ( __linux__ && __TBB_x86_32 && __GNUC__==3 && __GNUC_MINOR__==3 ) || defined(__SUNPRO_CC)
     // gcc 3.3 has known problem for 32-bit Linux, so only warn if there is a problem.
     // SUNPRO_CC does have this problem as well
     if( sizeof(T)==8 ) {
         if( sizeof(AlignmentChecker<T>)!=2*sizeof(tbb::atomic<T>) ) {
-            printf("Warning: alignment for atomic<%s> is wrong (known issue with gcc 3.3 and sunCC 5.9 2008/01/28 for IA32)\n",name);
+            REPORT("Warning: alignment for atomic<%s> is wrong (known issue with gcc 3.3 and sunCC 5.9 2008/01/28 for IA32)\n",name);
         }
     } else
 #endif /* ( __linux__ && __TBB_x86_32 && __GNUC__==3 && __GNUC_MINOR__==3 ) || defined(__SUNPRO_CC) */
@@ -242,12 +268,14 @@ void TestAtomicInteger( const char* name ) {
     for( int k=0; k<int(sizeof(long))*8-1; ++k ) {
         TestOperations<T>(T(1L<<k),T(~(1L<<k)),T(1-(1L<<k)));
         TestOperations<T>(T(-1L<<k),T(~(-1L<<k)),T(1-(-1L<<k)));
+        TestFetchAndAdd<T>(T(-1L<<k));
     }
     TestParallel<T>( name );
 }
+
 #if _MSC_VER && !defined(__INTEL_COMPILER)
-#pragma warning( pop )
-#endif /* _MSC_VER && !defined(__INTEL_COMPILER) */
+    #pragma warning( pop )
+#endif
 
 
 template<typename T>
@@ -263,7 +291,7 @@ void TestIndirection() {
     pointer = &item;
     for( int k=-10; k<=10; ++k ) {
         // Test various syntaxes for indirection to fields with non-zero offset.   
-        T value1, value2;
+        T value1=T(), value2=T();
         for( size_t j=0; j<sizeof(T); ++j ) {
             *(char*)&value1 = char(k^j);
             *(char*)&value2 = char(k^j*j);
@@ -277,41 +305,171 @@ void TestIndirection() {
     }
 }
 
+//! Test atomic<T*>
 template<typename T>
 void TestAtomicPointer() {
+    REMARK("testing atomic pointer (%d)\n",int(sizeof(T)));
     T array[1000];
     TestOperations<T*>(&array[500],&array[250],&array[750]);
-    TestOperations<void*>(&array[500],&array[250],&array[750]);
+    TestFetchAndAdd<T*>(&array[500]);
     TestIndirection<T>();
     TestParallel<T*>( "pointer" );
 }
 
-// Specialization for void*
-template<>
-void TestAtomicPointer<void*>() {
-    void* array[1000];
-    TestOperations<void*>(&array[500],&array[250],&array[750]);
-    TestParallel<void*>( "pointer" );
+//! Test atomic<Ptr> where Ptr is a pointer to a type of unknown size
+template<typename Ptr>
+void TestAtomicPointerToTypeOfUnknownSize( const char* name ) {
+    REMARK("testing atomic<%s>\n",name);
+    char array[1000];
+    TestOperations<Ptr>((Ptr)(void*)&array[500],(Ptr)(void*)&array[250],(Ptr)(void*)&array[750]);
+    TestParallel<Ptr>( name );
 }
 
 void TestAtomicBool() {
+    REMARK("testing atomic<bool>\n");
     TestOperations<bool>(true,true,false);
     TestOperations<bool>(false,false,true);
     TestParallel<bool>( "bool" );
 }
 
+enum Color {Red=0,Green=1,Blue=-1};
+
+void TestAtomicEnum() {
+    REMARK("testing atomic<Color>\n");
+    TestOperations<Color>(Red,Green,Blue);
+    TestParallel<Color>( "Color" );
+}
+
+#if !__TBB_FLOATING_POINT_BROKEN 
+template<typename T>
+void TestAtomicFloat( const char* name ) {
+    REMARK("testing atomic<%s>\n", name );
+    TestOperations<T>(0.5,3.25,10.75);
+    TestParallel<T>( name );
+}
+#endif /* !__TBB_FLOATING_POINT_BROKEN */
+
+const int numMaskedOperations = 100000;
+const int testSpaceSize = 8;
+int prime[testSpaceSize] = {3,5,7,11,13,17,19,23};
+
+#if _MSC_VER && !defined(__INTEL_COMPILER)
+    // "possible loss of data" warning suppressed again
+    #pragma warning( push )
+    #pragma warning( disable: 4244 )
+#endif
+
+template<typename T>
+class TestMaskedCAS_Body: NoAssign {
+    T*  test_space_uncontended;
+    T*  test_space_contended;
+public:   
+    TestMaskedCAS_Body( T* _space1, T* _space2 ) : test_space_uncontended(_space1), test_space_contended(_space2) {}
+    void operator()( int my_idx ) const {
+        using tbb::internal::__TBB_MaskedCompareAndSwap;
+        const T my_prime = T(prime[my_idx]);
+        T* const my_ptr = test_space_uncontended+my_idx;
+        T old_value=0;
+        for( int i=0; i<numMaskedOperations; ++i, old_value+=my_prime ){
+            T result;
+        // Test uncontended case
+            T new_value = old_value + my_prime;
+            // The following CAS should always fail
+            result = __TBB_MaskedCompareAndSwap<sizeof(T),T>(my_ptr,new_value,old_value-1);
+            ASSERT(result!=old_value-1, "masked CAS succeeded while it should fail");
+            ASSERT(result==*my_ptr, "masked CAS result mismatch with real value");
+            // The following one should succeed
+            result = __TBB_MaskedCompareAndSwap<sizeof(T),T>(my_ptr,new_value,old_value);
+            ASSERT(result==old_value && *my_ptr==new_value, "masked CAS failed while it should succeed");
+            // The following one should fail again
+            result = __TBB_MaskedCompareAndSwap<sizeof(T),T>(my_ptr,new_value,old_value);
+            ASSERT(result!=old_value, "masked CAS succeeded while it should fail");
+            ASSERT(result==*my_ptr, "masked CAS result mismatch with real value");
+        // Test contended case
+            for( int j=0; j<testSpaceSize; ++j ){
+                // try adding my_prime until success
+                T value;
+                do {
+                    value = test_space_contended[j];
+                    result = __TBB_MaskedCompareAndSwap<sizeof(T),T>(test_space_contended+j,value+my_prime,value);
+                } while( result!=value );
+            }
+        }
+    }
+};
+
+template<typename T>
+struct intptr_as_array_of
+{
+    static const int how_many_Ts = sizeof(intptr_t)/sizeof(T);
+    union {
+        intptr_t result;
+        T space[ how_many_Ts ];
+    };
+};
+
+template<typename T>
+intptr_t getCorrectUncontendedValue(int slot_idx) {
+    intptr_as_array_of<T> slot;
+    slot.result = 0;
+    for( int i=0; i<slot.how_many_Ts; ++i ) {
+        const T my_prime = T(prime[slot_idx*slot.how_many_Ts + i]);
+        for( int j=0; j<numMaskedOperations; ++j )
+            slot.space[i] += my_prime;
+    }
+    return slot.result;
+}
+
+template<typename T>
+intptr_t getCorrectContendedValue() {
+    intptr_as_array_of<T>  slot;
+    slot.result = 0;
+    for( int i=0; i<slot.how_many_Ts; ++i )
+        for( int primes=0; primes<testSpaceSize; ++primes )
+            for( int j=0; j<numMaskedOperations; ++j )
+                slot.space[i] += prime[primes];
+    return slot.result;
+}
+
+#if _MSC_VER && !defined(__INTEL_COMPILER)
+    #pragma warning( pop )
+#endif // warning 4244 is back again
+
+template<typename T>
+void TestMaskedCAS() {
+    REMARK("testing masked CAS<%d>\n",int(sizeof(T)));
+
+    const int num_slots = sizeof(T)*testSpaceSize/sizeof(intptr_t);
+    intptr_t arr1[num_slots+2]; // two more "canary" slots at boundaries
+    intptr_t arr2[num_slots+2];
+    for(int i=0; i<num_slots+2; ++i)
+        arr2[i] = arr1[i] = 0;
+    T* test_space_uncontended = (T*)(arr1+1);
+    T* test_space_contended = (T*)(arr2+1);
+
+    NativeParallelFor( testSpaceSize, TestMaskedCAS_Body<T>(test_space_uncontended, test_space_contended) );
+
+    ASSERT( arr1[0]==0 && arr1[num_slots+1]==0 && arr2[0]==0 && arr2[num_slots+1]==0 , "adjacent memory was overwritten" );
+    const intptr_t correctContendedValue = getCorrectContendedValue<T>();
+    for(int i=0; i<num_slots; ++i) {
+        ASSERT( arr1[i+1]==getCorrectUncontendedValue<T>(i), "unexpected value in an uncontended slot" );
+        ASSERT( arr2[i+1]==correctContendedValue, "unexpected value in a contended slot" );
+    }
+}
+
 template<unsigned N>
 class ArrayElement {
     char item[N];
 };
 
+__TBB_TEST_EXPORT
 int main( int argc, char* argv[] ) {
     ParseCommandLine( argc, argv );
 #if defined(__INTEL_COMPILER)||!defined(_MSC_VER)||_MSC_VER>=1400
     TestAtomicInteger<unsigned long long>("unsigned long long");
     TestAtomicInteger<long long>("long long");
 #else
-    printf("Warning: atomic<64-bits> not tested because of known problem in Microsoft compiler\n");
+    REPORT("Warning: atomic<64-bits> not tested because of known problem in Microsoft compiler\n");
 #endif /*defined(__INTEL_COMPILER)||!defined(_MSC_VER)||_MSC_VER>=1400 */
     TestAtomicInteger<unsigned long>("unsigned long");
     TestAtomicInteger<long>("long");
@@ -333,18 +491,21 @@ int main( int argc, char* argv[] ) {
     TestAtomicPointer<ArrayElement<6> >();
     TestAtomicPointer<ArrayElement<7> >();
     TestAtomicPointer<ArrayElement<8> >();
-    TestAtomicPointer<void*>();
+    TestAtomicPointerToTypeOfUnknownSize<IncompleteType*>( "IncompleteType*" );
+    TestAtomicPointerToTypeOfUnknownSize<void*>( "void*" );
     TestAtomicBool();
+    TestAtomicEnum();
+#if !__TBB_FLOATING_POINT_BROKEN 
+    TestAtomicFloat<float>("float");
+    TestAtomicFloat<double>("double");
+#endif /* !__TBB_FLOATING_POINT_BROKEN  */
     ASSERT( !ParallelError, NULL );
-    printf("done\n");
+    TestMaskedCAS<unsigned char>();
+    TestMaskedCAS<unsigned short>();
+    REPORT("done\n");
     return 0;
 }
 
-// Portions dependent on blocked_range.h are down here, so that preceding tests do not
-// accidentally depend upon it.
-
-#include "tbb/blocked_range.h"
-
 template<typename T>
 struct FlagAndMessage {
     //! 0 if message not set yet, 1 if message is set.
@@ -362,6 +523,13 @@ T special_sum(intptr_t arg1, intptr_t arg2) {
     return (T)((T)arg1 + arg2);
 }
 
+// The specialization for IncompleteType* is required
+// because pointer arithmetic (+) is impossible with IncompleteType*
+template<>
+IncompleteType* special_sum<IncompleteType*>(intptr_t arg1, intptr_t arg2) {
+    return (IncompleteType*)(arg1 + arg2);
+}
+
 // The specialization for void* is required
 // because pointer arithmetic (+) is impossible with void*
 template<>
@@ -379,7 +547,7 @@ bool special_sum<bool>(intptr_t arg1, intptr_t arg2) {
 volatile int One = 1;
  
 template<typename T>
-class HammerLoadAndStoreFence {
+class HammerLoadAndStoreFence: NoAssign {
     FlagAndMessage<T>* fam;
     const int n;
     const int p;
@@ -388,9 +556,8 @@ class HammerLoadAndStoreFence {
     mutable T accum;
 public:
     HammerLoadAndStoreFence( FlagAndMessage<T>* fam_, int n_, int p_, const char* name_, int trial_ ) : fam(fam_), n(n_), p(p_), trial(trial_), name(name_) {}
-    void operator()( const tbb::blocked_range<int>& range ) const {
+    void operator()( int k ) const {
         int one = One;
-        int k = range.begin();
         FlagAndMessage<T>* s = fam+k;
         FlagAndMessage<T>* s_next = fam + (k+1)%p;
         for( int i=0; i<n; ++i ) {
@@ -414,15 +581,15 @@ public:
                 }
                 if( flag ) {
                     if( flag!=(T)-1 ) {
-                        printf("ERROR: flag!=(T)-1 k=%d i=%d trial=%x type=%s (atomicity problem?)\n", k, i, trial, name );
+                        REPORT("ERROR: flag!=(T)-1 k=%d i=%d trial=%x type=%s (atomicity problem?)\n", k, i, trial, name );
                         ParallelError = true;
                     } 
                     if( message!=(T)-1 ) {
-                        printf("ERROR: message!=(T)-1 k=%d i=%d trial=%x type=%s (memory fence problem?)\n", k, i, trial, name );
+                        REPORT("ERROR: message!=(T)-1 k=%d i=%d trial=%x type=%s (memory fence problem?)\n", k, i, trial, name );
                         ParallelError = true;
                     }
-                    s->message = 0; 
-                    s->flag = 0;
+                    s->message = T(0); 
+                    s->flag = T(0);
                     // Set message and then the flag
                     if( trial&2 ) {
                         // COMPLICATED_ZERO here tempts compiler to sink store below setting of flag
@@ -455,7 +622,7 @@ void TestLoadAndStoreFences( const char* name ) {
             memset( fam, 0, p*sizeof(FlagAndMessage<T>) );
             fam->message = (T)-1;
             fam->flag = (T)-1;
-            NativeParallelFor( tbb::blocked_range<int>(0,p,1), HammerLoadAndStoreFence<T>( fam, 100, p, name, trial ) );
+            NativeParallelFor( p, HammerLoadAndStoreFence<T>( fam, 100, p, name, trial ) );
             for( int k=0; k<p; ++k ) {
                 ASSERT( fam[k].message==(k==0 ? (T)-1 : 0), "incomplete round-robin?" ); 
                 ASSERT( fam[k].flag==(k==0 ? (T)-1 : 0), "incomplete round-robin?" ); 
@@ -477,14 +644,14 @@ public:
         // 1. It has at least one 1 in most of its bytes.
         // 2. The bytes are typically different.
         // 3. When multiplied by any value <=127, the product does not overflow.
-        factor = 0;
+        factor = T(0);
         for( unsigned i=0; i<sizeof(T)*8-7; i+=7 ) 
-            factor |= (T)1<<i;
+            factor = T(factor | T(1)<<i);
      }
      //! Get ith member of set
      T get( int i ) const {
          // Create multiple of factor.  The & prevents overflow of the product.
-         return (i&0x7F)*factor;
+         return T((i&0x7F)*factor);
      }        
      //! True if set contains x
      bool contains( T x ) const {
@@ -512,23 +679,73 @@ public:
     bool contains( bool ) const {return true;}
 };
 
+#if !__TBB_FLOATING_POINT_BROKEN
+
+#if _MSC_VER==1500 && !defined(__INTEL_COMPILER)
+    // VS2008/VC9 seems to have an issue; limits pull in math.h
+    #pragma warning( push )
+    #pragma warning( disable: 4985 )
+#endif
+#include <limits> /* Need std::numeric_limits */
+#if _MSC_VER==1500 && !defined(__INTEL_COMPILER)
+    #pragma warning( pop )
+#endif
+
+//! Commonality inherited by specializations for floating-point types.
 template<typename T>
-class HammerAssignment {
+class SparseFloatSet: NoAssign {
+    const T epsilon;
+public:
+    SparseFloatSet() : epsilon(std::numeric_limits<T>::epsilon()) {}
+    T get( int i ) const {
+        return i==0 ? T(0) : 1/T((i&0x7F)+1);
+    }
+    bool contains( T x ) const {
+        if( x==T(0) ) {
+            return true;
+        } else {
+            int j = int(1/x+T(0.5));
+            if( 0<j && j<=128 ) {
+                T error = x*T(j)-T(1);
+                // In the calculation above, if x was indeed generated by method get, the error should be 
+                // at most epsilon, because x is off by at most 1/2 ulp from its infinitely precise value, 
+                // j is exact, and the multiplication incurs at most another 1/2 ulp of round-off error.
+                if( -epsilon<=error && error<=epsilon ) {
+                    return true;
+                } else {
+                    REPORT("Warning: excessive floating-point error encountered j=%d x=%.15g error=%.15g\n",j,x,error);
+                }
+            }
+            return false;
+        }
+    };
+};
+
+template<> 
+class SparseValueSet<float>: public SparseFloatSet<float> {};
+
+template<> 
+class SparseValueSet<double>: public SparseFloatSet<double> {};
+
+#endif /* !__TBB_FLOATING_POINT_BROKEN */
+
+template<typename T>
+class HammerAssignment: NoAssign {
     tbb::atomic<T>& x;
     const char* name;
     SparseValueSet<T> set;
 public:   
     HammerAssignment( tbb::atomic<T>& x_, const char* name_ ) : x(x_), name(name_) {}
-    void operator()( const tbb::blocked_range<int>& range ) const {
+    void operator()( int k ) const {
         const int n = 1000000;
-        if( range.begin() ) {
+        if( k ) {
             tbb::atomic<T> z;
             AssertSameType( z=x, z );    // Check that return type from assignment is correct
             for( int i=0; i<n; ++i ) {
                 // Read x atomically into z.
                 z = x;
                 if( !set.contains(z) ) {
-                    printf("ERROR: assignment of atomic<%s> is not atomic\n", name);
+                    REPORT("ERROR: assignment of atomic<%s> is not atomic\n", name);
                     ParallelError = true;
                     return;
                 }
@@ -545,11 +762,22 @@ public:
     }
 };
 
+// Compile-time check that a class method has the required signature.
+// Intended to check the assignment operator of tbb::atomic.
+template<typename T> void TestAssignmentSignature( T& (T::*)(const T&) ) {}
+
+#if _MSC_VER && !defined(__INTEL_COMPILER)
+    // Suppress "conditional expression is constant" warning.
+    #pragma warning( push )
+    #pragma warning( disable: 4127 )
+#endif
+
 template<typename T>
 void TestAssignment( const char* name ) {
+    TestAssignmentSignature( &tbb::atomic<T>::operator= );
     tbb::atomic<T> x;
-    x = 0;
-    NativeParallelFor( tbb::blocked_range<int>(0,2,1), HammerAssignment<T>( x, name ) );
+    x = T(0);
+    NativeParallelFor( 2, HammerAssignment<T>( x, name ) );
 #if __TBB_x86_32 && (__linux__ || __FreeBSD__ || _WIN32)
     if( sizeof(T)==8 ) {
         // Some compilers for IA-32 fail to provide 8-byte alignment of objects on the stack, 
@@ -566,14 +794,18 @@ void TestAssignment( const char* name ) {
         // y crosses 8-byte boundary if and only if x does not cross.
         tbb::atomic<T>& y = *reinterpret_cast<tbb::atomic<T>*>((reinterpret_cast<uintptr_t>(&raw_space[7+delta])&~7u) - delta);
         // Assertion checks that y really did end up somewhere inside "raw_space".
-        __TBB_ASSERT( raw_space<=reinterpret_cast<char*>(&y), "y starts before raw_space" );
-        __TBB_ASSERT( reinterpret_cast<char*>(&y+1) <= raw_space+sizeof(raw_space), "y starts after raw_space" );
-        y = 0;
-        NativeParallelFor( tbb::blocked_range<int>(0,2,1), HammerAssignment<T>( y, name ) );
+        ASSERT( raw_space<=reinterpret_cast<char*>(&y), "y starts before raw_space" );
+        ASSERT( reinterpret_cast<char*>(&y+1) <= raw_space+sizeof(raw_space), "y starts after raw_space" );
+        y = T(0);
+        NativeParallelFor( 2, HammerAssignment<T>( y, name ) );
     }
 #endif /* __TBB_x86_32 && (__linux__ || __FreeBSD__ || _WIN32) */
 }
 
+#if _MSC_VER && !defined(__INTEL_COMPILER)
+    #pragma warning( pop )
+#endif
+
 template<typename T>
 void TestParallel( const char* name ) {
     TestLoadAndStoreFences<T>(name);
