@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2008 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2009 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -26,10 +26,39 @@
     the GNU General Public License.
 */
 
-#ifndef TBB_PERFORMANCE_WARNINGS
-#define TBB_PERFORMANCE_WARNINGS 1
+#ifndef TBB_USE_PERFORMANCE_WARNINGS
+#define TBB_USE_PERFORMANCE_WARNINGS 1
 #endif
-#include "tbb/tbb_stddef.h"
+
+// Our tests usually include the header under test first.  But this test needs
+// to use the preprocessor to edit the identifier runtime_warning in concurrent_hash_map.h.
+// Hence we include a few other headers before doing the abusive edit.
+#include "tbb/tbb_stddef.h" /* Defines runtime_warning */
+#include "harness_assert.h" /* Prerequisite for defining hooked_warning */
+
+// The symbol internal::runtime_warning is normally an entry point into the TBB library.
+// Here for sake of testing, we define it to be hooked_warning, a routine peculiar to this unit test.
+#define runtime_warning hooked_warning
+
+static bool bad_hashing = false;
+
+namespace tbb { 
+    namespace internal {
+        static void hooked_warning( const char* /*format*/, ... ) {
+            ASSERT(bad_hashing, "unexpected bad hashing");
+        }
+    } // namespace internal
+} // namespace tbb
+
+#include "tbb/concurrent_hash_map.h"
+
+// Restore runtime_warning as an entry point into the TBB library.
+#undef runtime_warning
+
+// Test whether a sufficient set of headers were included to instantiate a concurernt_hash_map. OSS Bug #120 (& #130):
+// http://www.threadingbuildingblocks.org/bug_desc.php?id=120
+tbb::concurrent_hash_map<std::pair<std::pair<int,std::string>,const char*>,int> TestInstantiation;
+
 #include "tbb/parallel_for.h"
 #include "tbb/blocked_range.h"
 #include "tbb/atomic.h"
@@ -37,24 +66,13 @@
 #include "harness.h"
 #include "harness_allocator.h"
 
-// hook performance warning
-#define runtime_warning hooked_warning
-bool bad_hashing = false;
-namespace tbb { namespace internal {
-    void hooked_warning( const char* format, ... ) {
-        ASSERT(bad_hashing, NULL);
-    }
-}}// namespace tbb::internal
-
-#include "tbb/concurrent_hash_map.h"
-
 class MyException : public std::bad_alloc {
 public:
     virtual const char *what() const throw() { return "out of items limit"; }
     virtual ~MyException() throw() {}
 };
 
-/** Has tighly controlled interface so that we can verify
+/** Has tightly controlled interface so that we can verify
     that concurrent_hash_map uses only the required interface. */
 class MyKey {
 private:
@@ -82,7 +100,7 @@ protected:
         LIVE=0x1234,
         DEAD=0x5678
     } my_state;
-    void operator=( const MyData& );    // Deny acces
+    void operator=( const MyData& );    // Deny access
 public:
     MyData(int i = 0) {
         my_state = LIVE;
@@ -163,7 +181,7 @@ public:
     bool equal( const MyKey& j, const MyKey& k ) const {
         return j.key==k.key;
     }
-    unsigned long hash( const MyKey& k ) const {
+    unsigned long hash( const MyKey& ) const {
         return 1;
     }   
 };
@@ -177,15 +195,14 @@ template<typename MyTable>
 inline void CheckAllocator(MyTable &table, size_t expected_allocs, size_t expected_frees, bool exact = true) {
     size_t items_allocated = table.get_allocator().items_allocated, items_freed = table.get_allocator().items_freed;
     size_t allocations = table.get_allocator().allocations, frees = table.get_allocator().frees;
-    if(Verbose)
-        printf("checking allocators: items %u/%u, allocs %u/%u\n",
+    REMARK("checking allocators: items %u/%u, allocs %u/%u\n",
             unsigned(items_allocated), unsigned(items_freed), unsigned(allocations), unsigned(frees) );
     ASSERT( items_allocated == allocations, NULL); ASSERT( items_freed == frees, NULL);
     if(exact) {
         ASSERT( allocations == expected_allocs, NULL); ASSERT( frees == expected_frees, NULL);
     } else {
         ASSERT( allocations >= expected_allocs, NULL); ASSERT( frees >= expected_frees, NULL);
-        ASSERT( allocations == frees, NULL );
+        ASSERT( allocations - frees == expected_allocs - expected_frees, NULL );
     }
 }
 
@@ -225,7 +242,7 @@ struct Find {
         ASSERT( b==!a.empty(), NULL );
         if( b ) {
             if( !UseKey(i) )
-                printf("Line %d: unexpected key %d present\n",__LINE__,i);
+                REPORT("Line %d: unexpected key %d present\n",__LINE__,i);
             AssertSameType( &*a, static_cast<MyTable::value_type*>(0) );
             ASSERT( ca->second.value_of()==i*i, NULL );
             ASSERT( (*ca).second.value_of()==i*i, NULL );
@@ -235,7 +252,7 @@ struct Find {
                 (*ca).second.set_value( ~ca->second.value_of() );
         } else {
             if( UseKey(i) ) 
-                printf("Line %d: key %d missing\n",__LINE__,i);
+                REPORT("Line %d: key %d missing\n",__LINE__,i);
         }
     }
 };
@@ -314,7 +331,7 @@ struct InnerInsert {
 };
 
 template<typename Op, typename MyTable>
-class TableOperation {
+class TableOperation: NoAssign {
     MyTable& my_table;
 public:
     void operator()( const tbb::blocked_range<int>& range ) const {
@@ -326,19 +343,16 @@ public:
 
 template<typename Op, typename TableType>
 void DoConcurrentOperations( TableType& table, int n, const char* what, int nthread ) {
-    if( Verbose ) 
-        printf("testing %s with %d threads\n",what,nthread);
+    REMARK("testing %s with %d threads\n",what,nthread);
     tbb::tick_count t0 = tbb::tick_count::now();
     tbb::parallel_for( tbb::blocked_range<int>(0,n,100), TableOperation<Op,TableType>(table) );
     tbb::tick_count t1 = tbb::tick_count::now();
-    if( Verbose )
-        printf("time for %s = %g with %d threads\n",what,(t1-t0).seconds(),nthread);
+    REMARK("time for %s = %g with %d threads\n",what,(t1-t0).seconds(),nthread);
 }
 
 //! Test traversing the table with an iterator.
 void TraverseTable( MyTable& table, size_t n, size_t expected_size ) {
-    if( Verbose ) 
-        printf("testing traversal\n");
+    REMARK("testing traversal\n");
     size_t actual_size = table.size();
     ASSERT( actual_size==expected_size, NULL );
     size_t count = 0;
@@ -371,14 +385,14 @@ void TraverseTable( MyTable& table, size_t n, size_t expected_size ) {
     ASSERT( ci==const_table.end(), NULL );
     delete[] array;
     if( count!=expected_size ) {
-        printf("Line %d: count=%ld but should be %ld\n",__LINE__,long(count),long(expected_size));
+        REPORT("Line %d: count=%ld but should be %ld\n",__LINE__,long(count),long(expected_size));
     }
 }
 
 typedef tbb::atomic<unsigned char> AtomicByte;
 
 template<typename RangeType>
-struct ParallelTraverseBody {
+struct ParallelTraverseBody: NoAssign {
     const size_t n;
     AtomicByte* const array;
     ParallelTraverseBody( AtomicByte array_[], size_t n_ ) : 
@@ -398,7 +412,7 @@ void Check( AtomicByte array[], size_t n, size_t expected_size ) {
     if( expected_size )
         for( size_t k=0; k<n; ++k ) {
             if( array[k] != int(UseKey(k)) ) {
-                printf("array[%d]=%d != %d=UseKey(%d)\n",
+                REPORT("array[%d]=%d != %d=UseKey(%d)\n",
                        int(k), int(array[k]), int(UseKey(k)), int(k));
                 ASSERT(false,NULL);
             }
@@ -407,8 +421,7 @@ void Check( AtomicByte array[], size_t n, size_t expected_size ) {
 
 //! Test travering the tabel with a parallel range
 void ParallelTraverseTable( MyTable& table, size_t n, size_t expected_size ) {
-    if( Verbose ) 
-        printf("testing parallel traversal\n");
+    REMARK("testing parallel traversal\n");
     ASSERT( table.size()==expected_size, NULL );
     AtomicByte* array = new AtomicByte[n];
 
@@ -480,13 +493,13 @@ void TestInsertFindErase( int nthread ) {
 
 volatile int Counter;
 
-class AddToTable {
+class AddToTable: NoAssign {
     MyTable& my_table;
     const int my_nthread;
     const int my_m;
 public:
     AddToTable( MyTable& table, int nthread, int m ) : my_table(table), my_nthread(nthread), my_m(m) {}
-    void operator()( const  tbb::blocked_range<int>& r ) const {
+    void operator()( int ) const {
         for( int i=0; i<my_m; ++i ) {
             // Busy wait to synchronize threads
             int j = 0;
@@ -502,7 +515,7 @@ public:
             // Now all threads attempt to simultaneously insert a key.
             int k;
             {
-                MyTable::accessor a, b;
+                MyTable::accessor a;
                 MyKey key = MyKey::make(i);
                 if( my_table.insert( a, key ) ) 
                     a->second.set_value( 1 );
@@ -516,13 +529,13 @@ public:
     }
 };
 
-class RemoveFromTable {
+class RemoveFromTable: NoAssign {
     MyTable& my_table;
     const int my_nthread;
     const int my_m;
 public:
     RemoveFromTable( MyTable& table, int nthread, int m ) : my_table(table), my_nthread(nthread), my_m(m) {}
-    void operator()( const  tbb::blocked_range<int>& r ) const {
+    void operator()(int) const {
         for( int i=0; i<my_m; ++i ) {
             bool b;
             if(i&4) {
@@ -542,26 +555,23 @@ public:
 
 //! Test for memory leak in concurrent_hash_map (TR #153).
 void TestConcurrency( int nthread ) {
-    if( Verbose ) 
-        printf("testing multiple insertions/deletions of same key with %d threads\n", nthread);
+    REMARK("testing multiple insertions/deletions of same key with %d threads\n", nthread);
     {
         ASSERT( MyDataCount==0, NULL );
         MyTable table;
         const int m = 1000;
         Counter = 0;
         tbb::tick_count t0 = tbb::tick_count::now();
-        NativeParallelFor( tbb::blocked_range<int>(0,nthread,1), AddToTable(table,nthread,m) );
+        NativeParallelFor( nthread, AddToTable(table,nthread,m) );
         tbb::tick_count t1 = tbb::tick_count::now();
-        if( Verbose )
-            printf("time for %u insertions = %g with %d threads\n",unsigned(MyDataCount),(t1-t0).seconds(),nthread);
+        REMARK("time for %u insertions = %g with %d threads\n",unsigned(MyDataCount),(t1-t0).seconds(),nthread);
         ASSERT( MyDataCount==m, "memory leak detected" );
 
         EraseCount = 0;
         t0 = tbb::tick_count::now();
-        NativeParallelFor( tbb::blocked_range<int>(0,nthread,1), RemoveFromTable(table,nthread,m) );
+        NativeParallelFor( nthread, RemoveFromTable(table,nthread,m) );
         t1 = tbb::tick_count::now();
-        if( Verbose )
-            printf("time for %u deletions = %g with %d threads\n",unsigned(EraseCount),(t1-t0).seconds(),nthread);
+        REMARK("time for %u deletions = %g with %d threads\n",unsigned(EraseCount),(t1-t0).seconds(),nthread);
         ASSERT( MyDataCount==0, "memory leak detected" );
         ASSERT( EraseCount==m, "return value of erase() is broken" );
 
@@ -609,23 +619,23 @@ void TestRangeAssignment( Range2 r2 ) {
 //------------------------------------------------------------------------
 
 template<typename MyTable>
-void FillTable( MyTable& x, int n ) {
+static void FillTable( MyTable& x, int n ) {
     for( int i=1; i<=n; ++i ) {
-        MyKey key( MyKey::make(i) );
+        MyKey key( MyKey::make(-i) ); // hash values must not be specified in direct order
         typename MyTable::accessor a;
         bool b = x.insert(a,key); 
-        ASSERT(b,NULL); 
+        ASSERT(b, NULL);
         a->second.set_value( i*i );
     }
 }
 
 template<typename MyTable>
-void CheckTable( const MyTable& x, int n ) {
+static void CheckTable( const MyTable& x, int n ) {
     ASSERT( x.size()==size_t(n), "table is different size than expected" );
     ASSERT( x.empty()==(n==0), NULL );
     ASSERT( x.size()<=x.max_size(), NULL );
     for( int i=1; i<=n; ++i ) {
-        MyKey key( MyKey::make(i) );
+        MyKey key( MyKey::make(-i) );
         typename MyTable::const_accessor a;
         bool b = x.find(a,key); 
         ASSERT( b, NULL ); 
@@ -635,21 +645,20 @@ void CheckTable( const MyTable& x, int n ) {
     int key_sum = 0;
     for( typename MyTable::const_iterator i(x.begin()); i!=x.end(); ++i ) {
         ++count;
-        key_sum += i->first.value_of();
+        key_sum += -i->first.value_of();
     }
     ASSERT( count==n, NULL );
     ASSERT( key_sum==n*(n+1)/2, NULL );
 }
 
-void TestCopy() {
-    if( Verbose )
-        printf("testing copy\n");
+static void TestCopy() {
+    REMARK("testing copy\n");
     MyTable t1;
     for( int i=0; i<10000; i=(i<100 ? i+1 : i*3) ) {
         MyDataCount = 0;
 
-        FillTable( t1, i );
-        CheckTable(t1,i);
+        FillTable(t1,i);
+        // Do not call CheckTable(t1,i) before copying, it enforces rehashing
 
         MyTable t2(t1);
         // Check that copy constructor did not mangle source table.
@@ -674,8 +683,7 @@ void TestCopy() {
 }
 
 void TestAssignment() {
-    if( Verbose )
-        printf("testing assignment\n");
+    REMARK("testing assignment\n");
     for( int i=0; i<1000; i=(i<30 ? i+1 : i*5) ) {
         for( int j=0; j<1000; j=(j<30 ? j+1 : j*7) ) {
             MyTable t1;
@@ -683,11 +691,11 @@ void TestAssignment() {
             FillTable(t1,i);
             FillTable(t2,j);
             ASSERT( (t1 == t2) == (i == j), NULL );
-            CheckTable(t1,i);
             CheckTable(t2,j);
 
             MyTable& tref = t2=t1; 
             ASSERT( &tref==&t2, NULL );
+            ASSERT( t1 == t2, NULL );
             CheckTable(t1,i);
             CheckTable(t2,i);
 
@@ -705,8 +713,7 @@ void TestAssignment() {
 }
 
 void TestIteratorsAndRanges() {
-    if( Verbose )
-        printf("testing iterators compliance\n");
+    REMARK("testing iterators compliance\n");
     TestIteratorTraits<MyTable::iterator,MyTable::value_type>();
     TestIteratorTraits<MyTable::const_iterator,const MyTable::value_type>();
 
@@ -722,15 +729,13 @@ void TestIteratorsAndRanges() {
     ASSERT(v.equal_range(MyKey::make(-1)) == std::make_pair(v.end(), v.end()), NULL);
     ASSERT(u.equal_range(MyKey::make(-1)) == std::make_pair(u.end(), u.end()), NULL);
 
-    if( Verbose )
-        printf("testing ranges compliance\n");
+    REMARK("testing ranges compliance\n");
     TestRangeAssignment<MyTable::const_range_type>( u.range() );
     TestRangeAssignment<MyTable::const_range_type>( v.range() );
     TestRangeAssignment<MyTable::range_type>( v.range() );
     // doesn't compile as expected: TestRangeAssignment<typename V::range_type>( u.range() );
 
-    if( Verbose )
-        printf("testing construction and insertion from iterators range\n");
+    REMARK("testing construction and insertion from iterators range\n");
     FillTable( v, 1000 );
     MyTable2 t(v.begin(), v.end());
     CheckTable(t, 1000);
@@ -740,15 +745,14 @@ void TestIteratorsAndRanges() {
     t.insert(v.begin(), v.end()); // restore
     CheckTable(t, 1000);
 
-    if( Verbose )
-        printf("testing comparison\n");
+    REMARK("testing comparison\n");
     typedef tbb::concurrent_hash_map<MyKey,MyData2,YourHashCompare,MyAllocator> YourTable1;
     typedef tbb::concurrent_hash_map<MyKey,MyData2,YourHashCompare> YourTable2;
     YourTable1 t1;
     FillTable( t1, 10 );
     CheckTable(t1, 10 );
     YourTable2 t2(t1.begin(), t1.end());
-    MyKey key( MyKey::make(5) ); MyData2 data;
+    MyKey key( MyKey::make(-5) ); MyData2 data;
     ASSERT(t2.erase(key), NULL);
     YourTable2::accessor a;
     ASSERT(t2.insert(a, key), NULL);
@@ -766,8 +770,7 @@ void TestExceptions() {
         ctor_copy, op_assign, op_insert,
         all_methods
     };
-    if( Verbose )
-        printf("testing exception-safety guarantees\n");
+    REMARK("testing exception-safety guarantees\n");
     ThrowingTable src;
     FillTable( src, 1000 );
     ASSERT( MyDataCount==1000, NULL );
@@ -816,7 +819,7 @@ void TestExceptions() {
 
                 default:; // nothing to check here
                 }
-                if( Verbose ) printf("Exception %d: %s\t- ok ()\n", m, e.what());
+                REMARK("Exception %d: %s\t- ok ()\n", m, e.what());
             }
         }
     } catch(...) {
@@ -831,14 +834,14 @@ void TestExceptions() {
 
 #include "tbb/task_scheduler_init.h"
 
-//! Test driver
+__TBB_TEST_EXPORT
 int main( int argc, char* argv[] ) {
     // Default minimum number of threads is 1.
     MinThread = 1;
 
     ParseCommandLine(argc,argv);
     if( MinThread<0 ) {
-        printf("ERROR: must use at least one thread\n");
+        REPORT("ERROR: must use at least one thread\n");
         exit(1);
     }
 
@@ -847,7 +850,9 @@ int main( int argc, char* argv[] ) {
     TestCopy();
     TestAssignment();
     TestIteratorsAndRanges();
+#if !__TBB_EXCEPTION_HANDLING_TOTALLY_BROKEN
     TestExceptions();
+#endif
 
     // Do concurrency tests.
     for( int nthread=MinThread; nthread<=MaxThread; ++nthread ) {
@@ -856,11 +861,10 @@ int main( int argc, char* argv[] ) {
         TestConcurrency( nthread );
     }
     // check linking
-    #undef runtime_warning
     if(bad_hashing) { //should be false
         tbb::internal::runtime_warning("none\nERROR: it must not be executed");
     }
 
-    printf("done\n");
+    REPORT("done\n");
     return 0;
 }
