@@ -44,7 +44,7 @@ class Foo {
 public:
     int thread_id;
     int serial;
-    Foo() : state(LIVE) {
+    Foo() : state(LIVE), thread_id(0), serial(0) {
         ++FooConstructed;
     }
     Foo( const Foo& item ) : state(LIVE) {
@@ -132,6 +132,16 @@ static tbb::atomic<long> PopKind[3];
 
 const int M = 10000;
 
+#if TBB_DEPRECATED
+#define CALL_BLOCKING_POP(q,v) (q)->pop(v)
+#define CALL_TRY_POP(q,v,i) (((i)&0x2)?q->try_pop(v):q->pop_if_present(v))
+#define SIZE() size()
+#else
+#define CALL_BLOCKING_POP(q,v) while( !(q)->try_pop(v) ) __TBB_Yield()
+#define CALL_TRY_POP(q,v,i) q->try_pop(v)
+#define SIZE() unsafe_size()
+#endif
+
 struct Body: NoAssign {
     tbb::concurrent_queue<Foo>* queue;
     const int nthread;
@@ -149,7 +159,7 @@ struct Body: NoAssign {
             f.serial = 0xDEAD;
             bool prepopped = false;
             if( j&1 ) {
-                prepopped = queue->pop_if_present(f);
+                prepopped = CALL_TRY_POP(queue,f,j);
                 ++pop_kind[prepopped];
             }
             Foo g;
@@ -157,7 +167,7 @@ struct Body: NoAssign {
             g.serial = j+1;
             queue->push( g );
             if( !prepopped ) {
-                queue->pop(f);
+                CALL_BLOCKING_POP(queue,f);
                 ++pop_kind[2];
             }
             ASSERT( f.thread_id<=nthread, NULL );
@@ -171,10 +181,13 @@ struct Body: NoAssign {
     }
 };
 
-void TestPushPop( int prefill, ptrdiff_t capacity, int nthread ) {
+void TestPushPop( size_t prefill, ptrdiff_t capacity, int nthread ) {
     ASSERT( nthread>0, "nthread must be positive" );
-    if( prefill+1>=capacity )
+#if TBB_DEPRECATED
+    ptrdiff_t signed_prefill = ptrdiff_t(prefill);
+    if( signed_prefill+1>=capacity )
         return;
+#endif
     bool success = false;
     for( int k=0; k<3; ++k )
         PopKind[k] = 0;
@@ -183,37 +196,43 @@ void TestPushPop( int prefill, ptrdiff_t capacity, int nthread ) {
         FooDestroyed = 0;
         Body body(nthread);
         tbb::concurrent_queue<Foo> queue;
+#if TBB_DEPRECATED
         queue.set_capacity( capacity );
+#endif
         body.queue = &queue;
-        for( int i=0; i<prefill; ++i ) {
+        for( size_t i=0; i<prefill; ++i ) {
             Foo f;
             f.thread_id = nthread;
-            f.serial = 1+i;
+            f.serial = 1+int(i);
             queue.push(f);
-            ASSERT( queue.size()==i+1, NULL );
+            ASSERT( unsigned(queue.SIZE())==i+1, NULL );
             ASSERT( !queue.empty(), NULL );
         }
         tbb::tick_count t0 = tbb::tick_count::now();
         NativeParallelFor( nthread, body );
         tbb::tick_count t1 = tbb::tick_count::now();
+#if !__TBB_FLOATING_POINT_BROKEN
         double timing = (t1-t0).seconds();
-        if( Verbose )
-            printf("prefill=%d capacity=%d time = %g = %g nsec/operation\n", prefill, int(capacity), timing, timing/(2*M*nthread)*1.E9);
+        REMARK("prefill=%d capacity=%d time = %g = %g nsec/operation\n", int(prefill), int(capacity), timing, timing/(2*M*nthread)*1.E9);
+#else
+        ((void)capacity);   // touch it to suppress the warning
+#endif /* !__TBB_FLOATING_POINT_BROKEN */
         int sum = 0;
         for( int k=0; k<nthread; ++k )
             sum += Sum[k];
-        int expected = nthread*((M-1)*M/2) + ((prefill-1)*prefill)/2;
-        for( int i=prefill; --i>=0; ) {
+        int expected = int(nthread*((M-1)*M/2) + ((prefill-1)*prefill)/2);
+        for( int i=int(prefill); --i>=0; ) {
             ASSERT( !queue.empty(), NULL );
             Foo f;
-            queue.pop(f);
-            ASSERT( queue.size()==i, NULL );
+            bool result = queue.try_pop(f);
+            ASSERT( result, NULL );
+            ASSERT( int(queue.SIZE())==i, NULL );
             sum += f.serial-1;
         }
         ASSERT( queue.empty(), NULL );
-        ASSERT( queue.size()==0, NULL );
+        ASSERT( queue.SIZE()==0, NULL );
         if( sum!=expected )
-            printf("sum=%d expected=%d\n",sum,expected);
+            REPORT("sum=%d expected=%d\n",sum,expected);
         ASSERT( FooConstructed==FooDestroyed, NULL );
         // TODO: checks by counting allocators
 
@@ -234,12 +253,12 @@ void TestPushPop( int prefill, ptrdiff_t capacity, int nthread ) {
                 if( PopKind[k]<min_requirement ) {
                     if( trial>=max_trial ) {
                         if( Verbose )
-                            printf("Warning: %d threads had only %ld pop_if_present operations %s after %d trials (expected at least %d). "
+                            REPORT("Warning: %d threads had only %ld pop_if_present operations %s after %d trials (expected at least %d). "
                                "This problem may merely be unlucky scheduling. "
                                "Investigate only if it happens repeatedly.\n",
                                nthread, long(PopKind[k]), k==0?"failed":"succeeded", max_trial, min_requirement);
                         else
-                            printf("Warning: the number of %s pop_if_present operations is less than expected for %d threads. Investigate if it happens repeatedly.\n",
+                            REPORT("Warning: the number of %s pop_if_present operations is less than expected for %d threads. Investigate if it happens repeatedly.\n",
                                k==0?"failed":"succeeded", nthread ); 
 
                     } else {
@@ -378,6 +397,14 @@ bool operator==(const BarEx& bar1, const BarEx& bar2) {
     return bar1.my_id==bar2.my_id && bar1.my_tilda_id==bar2.my_tilda_id;
 }
 
+#if TBB_DEPRECATED
+#define CALL_BEGIN(q,i) (((i)&0x1)?q.begin():q.unsafe_begin())
+#define CALL_END(q,i)   (((i)&0x1)?q.end():q.unsafe_end())
+#else
+#define CALL_BEGIN(q,i) q.unsafe_begin()
+#define CALL_END(q,i)   q.unsafe_end()
+#endif
+
 void TestConstructors ()
 {
     tbb::concurrent_queue<Bar> src_queue;
@@ -388,12 +415,12 @@ void TestConstructors ()
     for( size_t size=0; size<1001; ++size ) {
         for( size_t i=0; i<size; ++i )
             src_queue.push(Bar(i+(i^size)));
-        tbb::concurrent_queue<Bar>::const_iterator sqb(src_queue.begin());
-        tbb::concurrent_queue<Bar>::const_iterator sqe(src_queue.end());
+        tbb::concurrent_queue<Bar>::const_iterator sqb( CALL_BEGIN(src_queue,size) );
+        tbb::concurrent_queue<Bar>::const_iterator sqe( CALL_END(src_queue,size));
 
         tbb::concurrent_queue<Bar> dst_queue(sqb, sqe);
 
-        ASSERT(src_queue.size()==dst_queue.size(), "different size");
+        ASSERT(src_queue.SIZE()==dst_queue.SIZE(), "different size");
 
         src_queue.clear();
     }
@@ -408,12 +435,12 @@ void TestConstructors ()
 
         tbb::concurrent_queue<Bar> dst_queue2(sab, sae);
 
-        ASSERT( int(size)==dst_queue2.size(), NULL );
+        ASSERT( size==unsigned(dst_queue2.SIZE()), NULL );
         ASSERT( sab==BarIterator(bar_array+0), NULL );
         ASSERT( sae==BarIterator(bar_array+size), NULL );
 
-        dqb = dst_queue2.begin();
-        dqe = dst_queue2.end();
+        dqb = CALL_BEGIN(dst_queue2,size);
+        dqe = CALL_END(dst_queue2,size);
         BarIterator v_iter(sab);
         for( ; dqb != dqe; ++dqb, ++v_iter )
             ASSERT( *dqb == *v_iter, "unexpected element" );
@@ -423,36 +450,36 @@ void TestConstructors ()
     src_queue.clear();
 
     tbb::concurrent_queue<Bar> dst_queue3( src_queue );
-    ASSERT( src_queue.size()==dst_queue3.size(), NULL );
-    ASSERT( 0==dst_queue3.size(), NULL );
+    ASSERT( src_queue.SIZE()==dst_queue3.SIZE(), NULL );
+    ASSERT( 0==dst_queue3.SIZE(), NULL );
 
     int k=0;
     for( size_t i=0; i<1001; ++i ) {
         Bar tmp_bar;
         src_queue.push(Bar(++k));
         src_queue.push(Bar(++k));
-        src_queue.pop(tmp_bar);
+        src_queue.try_pop(tmp_bar);
 
         tbb::concurrent_queue<Bar> dst_queue4( src_queue );
 
-        ASSERT( src_queue.size()==dst_queue4.size(), NULL );
+        ASSERT( src_queue.SIZE()==dst_queue4.SIZE(), NULL );
 
-        dqb = dst_queue4.begin();
-        dqe = dst_queue4.end();
-        iter = src_queue.begin();
+        dqb = CALL_BEGIN(dst_queue4,i);
+        dqe = CALL_END(dst_queue4,i);
+        iter = CALL_BEGIN(src_queue,i);
 
         for( ; dqb != dqe; ++dqb, ++iter )
             ASSERT( *dqb == *iter, "unexpected element" );
 
-        ASSERT( iter==src_queue.end(), "different size?" );
+        ASSERT( iter==CALL_END(src_queue,i), "different size?" );
     }
 
     tbb::concurrent_queue<Bar> dst_queue5( src_queue );
 
-    ASSERT( src_queue.size()==dst_queue5.size(), NULL );
-    dqb = dst_queue5.begin();
-    dqe = dst_queue5.end();
-    iter = src_queue.begin();
+    ASSERT( src_queue.SIZE()==dst_queue5.SIZE(), NULL );
+    dqb = dst_queue5.unsafe_begin();
+    dqe = dst_queue5.unsafe_end();
+    iter = src_queue.unsafe_begin();
     for( ; dqb != dqe; ++dqb, ++iter )
         ASSERT( *dqb == *iter, "unexpected element" );
 
@@ -460,25 +487,28 @@ void TestConstructors ()
         Bar tmp_bar;
         src_queue.push(Bar(i+1000));
         src_queue.push(Bar(i+1000));
-        src_queue.pop(tmp_bar);
+        src_queue.try_pop(tmp_bar);
 
         dst_queue5.push(Bar(i+1000));
         dst_queue5.push(Bar(i+1000));
-        dst_queue5.pop(tmp_bar);
+        dst_queue5.try_pop(tmp_bar);
     }
 
-    ASSERT( src_queue.size()==dst_queue5.size(), NULL );
-    dqb = dst_queue5.begin();
-    dqe = dst_queue5.end();
-    iter = src_queue.begin();
+    ASSERT( src_queue.SIZE()==dst_queue5.SIZE(), NULL );
+    dqb = dst_queue5.unsafe_begin();
+    dqe = dst_queue5.unsafe_end();
+    iter = src_queue.unsafe_begin();
     for( ; dqb != dqe; ++dqb, ++iter )
         ASSERT( *dqb == *iter, "unexpected element" );
-    ASSERT( iter==src_queue.end(), "different size?" );
+    ASSERT( iter==src_queue.unsafe_end(), "different size?" );
 
-#if __GLIBC__==2&&__GLIBC_MINOR__==3
-    printf("Warning: Part of the construcor test is skipped due to a known issue.\n");
+#if __TBB_EXCEPTION_HANDLING_BROKEN || __TBB_PLACEMENT_NEW_EXCEPTION_SAFETY_BROKEN
+    REPORT("Warning: Part of the constructor test is skipped due to a known issue.\n");
 #else
     k = 0;
+#if TBB_DEPRECATED==0
+    unsigned
+#endif
     int n_elements=0;
     tbb::concurrent_queue<BarEx> src_queue_ex;
     for( size_t size=0; size<1001; ++size ) {
@@ -497,22 +527,22 @@ void TestConstructors ()
         } catch (...) {
         }
         ++k;
-        src_queue_ex.pop(tmp_bar_ex);
+        src_queue_ex.try_pop(tmp_bar_ex);
         n_elements += (n_successful_pushes - 1);
-        ASSERT( src_queue_ex.size()==n_elements, NULL);
+        ASSERT( src_queue_ex.SIZE()==n_elements, NULL);
 
         BarEx::set_mode( BarEx::COPY_CONSTRUCT );
         tbb::concurrent_queue<BarEx> dst_queue_ex( src_queue_ex );
 
-        ASSERT( src_queue_ex.size()==dst_queue_ex.size(), NULL );
+        ASSERT( src_queue_ex.SIZE()==dst_queue_ex.SIZE(), NULL );
 
-        tbb::concurrent_queue<BarEx>::const_iterator dqb_ex  = dst_queue_ex.begin();
-        tbb::concurrent_queue<BarEx>::const_iterator dqe_ex  = dst_queue_ex.end();
-        tbb::concurrent_queue<BarEx>::const_iterator iter_ex = src_queue_ex.begin();
+        tbb::concurrent_queue<BarEx>::const_iterator dqb_ex  = CALL_BEGIN(dst_queue_ex, size);
+        tbb::concurrent_queue<BarEx>::const_iterator dqe_ex  = CALL_END(dst_queue_ex, size);
+        tbb::concurrent_queue<BarEx>::const_iterator iter_ex = CALL_BEGIN(src_queue_ex, size);
 
         for( ; dqb_ex != dqe_ex; ++dqb_ex, ++iter_ex )
             ASSERT( *dqb_ex == *iter_ex, "unexpected element" );
-        ASSERT( iter_ex==src_queue_ex.end(), "different size?" );
+        ASSERT( iter_ex==CALL_END(src_queue_ex,size), "different size?" );
     }
 #endif
 }
@@ -574,17 +604,17 @@ void TestIterator() {
     tbb::concurrent_queue<Foo> queue;
     tbb::concurrent_queue<Foo>& const_queue = queue;
     for( int j=0; j<500; ++j ) {
-        TestIteratorAux( queue.begin(), queue.end(), j );
-        TestIteratorAux( const_queue.begin(), const_queue.end(), j );
-        TestIteratorAux( const_queue.begin(), queue.end(), j );
-        TestIteratorAux( queue.begin(), const_queue.end(), j );
+        TestIteratorAux( CALL_BEGIN(queue,j)      , CALL_END(queue,j)      , j );
+        TestIteratorAux( CALL_BEGIN(const_queue,j), CALL_END(const_queue,j), j );
+        TestIteratorAux( CALL_BEGIN(const_queue,j), CALL_END(queue,j)      , j );
+        TestIteratorAux( CALL_BEGIN(queue,j)      , CALL_END(const_queue,j), j );
         Foo f;
         f.serial = j+1;
         queue.push(f);
     }
-    TestIteratorAssignment<tbb::concurrent_queue<Foo>::const_iterator>( const_queue.begin() );
-    TestIteratorAssignment<tbb::concurrent_queue<Foo>::const_iterator>( queue.begin() );
-    TestIteratorAssignment<tbb::concurrent_queue<Foo>::iterator>( queue.begin() );
+    TestIteratorAssignment<tbb::concurrent_queue<Foo>::const_iterator>( const_queue.unsafe_begin() );
+    TestIteratorAssignment<tbb::concurrent_queue<Foo>::const_iterator>( queue.unsafe_begin() );
+    TestIteratorAssignment<tbb::concurrent_queue<Foo>::iterator>( queue.unsafe_begin() );
     TestIteratorTraits<tbb::concurrent_queue<Foo>::const_iterator, const Foo>();
     TestIteratorTraits<tbb::concurrent_queue<Foo>::iterator, Foo>();
 }
@@ -604,11 +634,15 @@ void TestConcurrentQueueType() {
 template<typename T>
 void TestEmptyQueue() {
     const tbb::concurrent_queue<T> queue;
-    ASSERT( queue.size()==0, NULL );
+    ASSERT( queue.SIZE()==0, NULL );
+#if TBB_DEPRECATED
     ASSERT( queue.capacity()>0, NULL );
     ASSERT( size_t(queue.capacity())>=size_t(-1)/(sizeof(void*)+sizeof(T)), NULL );
+#endif
 }
 
+#if TBB_DEPRECATED
+#define CALL_TRY_PUSH(q,f,i) (((i)&0x1)?(q).push_if_not_full(f):(q).try_push(f))
 void TestFullQueue() {
     for( int n=0; n<10; ++n ) {
         FooConstructed = 0;
@@ -618,7 +652,7 @@ void TestFullQueue() {
         for( int i=0; i<=n; ++i ) {
             Foo f;
             f.serial = i;
-            bool result = queue.push_if_not_full( f );
+            bool result = CALL_TRY_PUSH(queue, f, i );
             ASSERT( result==(i<n), NULL );
         }
         for( int i=0; i<=n; ++i ) {
@@ -630,42 +664,52 @@ void TestFullQueue() {
         ASSERT( FooConstructed==FooDestroyed, NULL );
     }
 }
+#endif /* if TBB_DEPRECATED */
+
+#if TBB_DEPRECATED
+#define CALL_PUSH_IF_NOT_FULL(q,v,i) (((i)&0x1)?q.push_if_not_full(v):(q.push(v), true))
+#else
+#define CALL_PUSH_IF_NOT_FULL(q,v,i) (q.push(v), true)
+#endif
 
 void TestClear() {
     FooConstructed = 0;
     FooDestroyed = 0;
-    const int n=5;
-    const int q_capacity=10;
+    const unsigned int n=5;
         
     tbb::concurrent_queue<Foo> queue;
+#if TBB_DEPRECATED
+    const int q_capacity=10;
     queue.set_capacity(q_capacity);
-    for( int i=0; i<n; ++i ) {
+#endif
+    for( size_t i=0; i<n; ++i ) {
         Foo f;
-        f.serial = i;
-        bool result = queue.push_if_not_full( f );
+        f.serial = int(i);
+        bool result = CALL_PUSH_IF_NOT_FULL(queue, f, i);
         ASSERT( result, NULL );
     }
-    ASSERT( queue.size()==n, NULL );
+    ASSERT( unsigned(queue.SIZE())==n, NULL );
     queue.clear();
-    ASSERT( queue.size()==0, NULL );
-    for( int i=0; i<n; ++i ) {
+    ASSERT( queue.SIZE()==0, NULL );
+    for( size_t i=0; i<n; ++i ) {
         Foo f;
-        f.serial = i;
-        bool result = queue.push_if_not_full( f );
+        f.serial = int(i);
+        bool result = CALL_PUSH_IF_NOT_FULL(queue, f, i);
         ASSERT( result, NULL );
     }
-    ASSERT( queue.size()==n, NULL );
+    ASSERT( unsigned(queue.SIZE())==n, NULL );
     queue.clear();
-    ASSERT( queue.size()==0, NULL );
-    for( int i=0; i<n; ++i ) {
+    ASSERT( queue.SIZE()==0, NULL );
+    for( size_t i=0; i<n; ++i ) {
         Foo f;
-        f.serial = i;
-        bool result = queue.push_if_not_full( f );
+        f.serial = int(i);
+        bool result = CALL_PUSH_IF_NOT_FULL(queue, f, i);
         ASSERT( result, NULL );
     }
-    ASSERT( queue.size()==n, NULL );
+    ASSERT( unsigned(queue.SIZE())==n, NULL );
 }
 
+#if TBB_DEPRECATED
 template<typename T>
 struct TestNegativeQueueBody: NoAssign {
     tbb::concurrent_queue<T>& queue;
@@ -699,6 +743,7 @@ void TestNegativeQueue( int nthread ) {
     tbb::concurrent_queue<T> queue;
     NativeParallelFor( nthread, TestNegativeQueueBody<T>(queue,nthread) );
 }
+#endif /* if TBB_DEPRECATED */
 
 void TestExceptions() {
     typedef static_counting_allocator<std::allocator<FooEx>, size_t> allocator_t;
@@ -710,8 +755,7 @@ void TestExceptions() {
         m_pop
     };  
 
-    if( Verbose )
-        printf("Testing exception safety\n");
+    REMARK("Testing exception safety\n");
     // verify 'clear()' on exception; queue's destructor calls its clear()
     {
         concur_queue_t queue_clear;
@@ -724,8 +768,7 @@ void TestExceptions() {
             // TODO: some assert here?
         }
     }
-    if( Verbose )
-        printf("... queue destruction test passed\n");
+    REMARK("... queue destruction test passed\n");
 
     try {
         int n_pushed=0, n_popped=0;
@@ -751,7 +794,7 @@ void TestExceptions() {
                             n_popped=0;
                             for( int k=0; k<n_pushed; k++ ) {
                                 FooEx elt;
-                                queue_test.pop( elt );
+                                queue_test.try_pop( elt );
                                 n_popped++;
                             }
                             n_pushed = 0;
@@ -762,7 +805,7 @@ void TestExceptions() {
                 } catch ( Foo_exception & ) {
                     switch(m) {
                     case m_push: {
-                                ASSERT( queue_test.size()==n_pushed, "incorrect queue size" );
+                                ASSERT( ptrdiff_t(queue_test.SIZE())==n_pushed, "incorrect queue size" );
                                 long tc = MaxFooCount;
                                 MaxFooCount = 0;
                                 for( int k=0; k<(int)tc; k++ ) {
@@ -775,23 +818,24 @@ void TestExceptions() {
                     case m_pop:
                             MaxFooCount = 0; // disable exception
                             n_pushed -= (n_popped+1); // including one that threw an exception
+                            ASSERT( n_pushed>=0, "n_pushed cannot be less than 0" );
                             for( int k=0; k<1000; k++ ) {
                                 queue_test.push( FooEx() );
                                 n_pushed++;
                             }
                             ASSERT( !queue_test.empty(), "queue must not be empty" );
-                            ASSERT( queue_test.size()==n_pushed, "queue size must be equal to n pushed" );
+                            ASSERT( ptrdiff_t(queue_test.SIZE())==n_pushed, "queue size must be equal to n pushed" );
                             for( int k=0; k<n_pushed; k++ ) {
                                 FooEx elt;
-                                queue_test.pop( elt );
+                                queue_test.try_pop( elt );
                             }
                             ASSERT( queue_test.empty(), "queue must be empty" );
-                            ASSERT( queue_test.size()==0, "queue must be empty" );
+                            ASSERT( queue_test.SIZE()==0, "queue must be empty" );
                             break;
                     }
                 } catch ( std::bad_alloc & ) {
                     allocator_char_t::set_limits(); // disable exception from allocator
-                    size_t size = queue_test.size();
+                    size_t size = queue_test.SIZE();
                     switch(m) {
                     case m_push:
                             ASSERT( size>0, "incorrect queue size");
@@ -801,8 +845,7 @@ void TestExceptions() {
                             break;
                     }
                 }
-                if( Verbose )
-                    printf("... for t=%d and m=%d, exception test passed\n", t, m);
+                REMARK("... for t=%d and m=%d, exception test passed\n", t, m);
             }
         }
     } catch(...) {
@@ -810,6 +853,7 @@ void TestExceptions() {
     }
 }
 
+__TBB_TEST_EXPORT
 int main( int argc, char* argv[] ) {
     // Set default for minimum number of threads.
     MinThread = 1;
@@ -817,7 +861,9 @@ int main( int argc, char* argv[] ) {
 
     TestEmptyQueue<char>();
     TestEmptyQueue<Foo>();
+#if TBB_DEPRECATED
     TestFullQueue();
+#endif
     TestClear();
     TestConcurrentQueueType();
     TestIterator();
@@ -825,8 +871,10 @@ int main( int argc, char* argv[] ) {
 
     // Test concurrent operations
     for( int nthread=MinThread; nthread<=MaxThread; ++nthread ) {
+#if TBB_DEPRECATED
         TestNegativeQueue<Foo>(nthread);
-        for( int prefill=0; prefill<64; prefill+=(1+prefill/3) ) {
+#endif
+        for( size_t prefill=0; prefill<64; prefill+=(1+prefill/3) ) {
             TestPushPop(prefill,ptrdiff_t(-1),nthread);
             TestPushPop(prefill,ptrdiff_t(1),nthread);
             TestPushPop(prefill,ptrdiff_t(2),nthread);
@@ -834,11 +882,11 @@ int main( int argc, char* argv[] ) {
             TestPushPop(prefill,ptrdiff_t(100),nthread);
         }
     }
-#if __GLIBC__==2&&__GLIBC_MINOR__==3
-    printf("Warning: Exception safety test is skipped due to a known issue.\n");
+#if __TBB_EXCEPTION_HANDLING_BROKEN
+    REPORT("Warning: Exception safety test is skipped due to a known issue.\n");
 #else
     TestExceptions();
 #endif
-    printf("done\n");
+    REPORT("done\n");
     return 0;
 }
