@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2008 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2009 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -27,16 +27,18 @@
 */
 
 #include "tbb/concurrent_vector.h"
+#include "tbb/tbb_allocator.h"
 #include "tbb/cache_aligned_allocator.h"
 #include "tbb/tbb_exception.h"
 #include <cstdio>
 #include <cstdlib>
 #include <stdexcept>
+#include "harness_report.h"
 #include "harness_assert.h"
 #include "harness_allocator.h"
 
 static bool known_issue_verbose = false;
-#define KNOWN_ISSUE(msg) if(!known_issue_verbose) known_issue_verbose = true, printf(msg)
+#define KNOWN_ISSUE(msg) if(!known_issue_verbose) known_issue_verbose = true, REPORT(msg)
 
 tbb::atomic<long> FooCount;
 long MaxFooCount = 0;
@@ -51,63 +53,73 @@ public:
     virtual ~Foo_exception() throw() {}
 };
 
+static const int initial_value_of_bar = 42;
 struct Foo {
     int my_bar;
 public:
     enum State {
         ZeroInitialized=0,
-        DefaultInitialized=0x1234,
-        CopyInitialized=0x8765,
-        Destroyed=0x5678
+        DefaultInitialized=0xDEFAUL,
+        CopyInitialized=0xC0314,
+        Destroyed=0xDEADF00
     } state;
+    bool is_valid() const {
+        return state==DefaultInitialized||state==CopyInitialized;
+    }
+    bool is_valid_or_zero() const {
+        return is_valid()||(state==ZeroInitialized && !my_bar);
+    }
     int& zero_bar() {
-        ASSERT( state==DefaultInitialized||state==CopyInitialized||state==ZeroInitialized, NULL );
+        ASSERT( is_valid_or_zero(), NULL );
         return my_bar;
     }
     int& bar() {
-        ASSERT( state==DefaultInitialized||state==CopyInitialized, NULL );
+        ASSERT( is_valid(), NULL );
         return my_bar;
     }
     int bar() const {
-        ASSERT( state==DefaultInitialized||state==CopyInitialized, NULL );
+        ASSERT( is_valid(), NULL );
         return my_bar;
     }
-    static const int initial_value_of_bar = 42;
-    Foo() {
-        state = DefaultInitialized;
+    Foo( int bar = initial_value_of_bar ) {
+        my_bar = bar;
         if(MaxFooCount && FooCount >= MaxFooCount)
             throw Foo_exception();
-        ++FooCount;
-        my_bar = initial_value_of_bar;
+        FooCount++;
+        state = DefaultInitialized;
     }
     Foo( const Foo& foo ) {
-        state = CopyInitialized;
+        my_bar = foo.my_bar;
+        ASSERT( foo.is_valid_or_zero(), "bad source for copy" );
         if(MaxFooCount && FooCount >= MaxFooCount)
             throw Foo_exception();
-        ++FooCount;
-        my_bar = foo.my_bar;
+        FooCount++;
+        state = CopyInitialized;
     }
     ~Foo() {
-        ASSERT( state==DefaultInitialized||state==CopyInitialized||(state==ZeroInitialized && !my_bar), NULL );
-        state = Destroyed;
+        ASSERT( is_valid_or_zero(), NULL );
         my_bar = ~initial_value_of_bar;
-        --FooCount;
+        if(state != ZeroInitialized) --FooCount;
+        state = Destroyed;
     }
     bool operator==(const Foo &f) const { return my_bar == f.my_bar; }
     bool operator<(const Foo &f) const { return my_bar < f.my_bar; }
     bool is_const() const {return true;}
     bool is_const() {return false;}
-private:
+protected:
     char reserve[1];
+    void operator=( const Foo& ) {}
 };
 
 class FooWithAssign: public Foo {
 public:
     void operator=( const FooWithAssign& x ) {
-        ASSERT( x.state==DefaultInitialized||x.state==CopyInitialized, NULL );
-        ASSERT( state==DefaultInitialized||state==CopyInitialized, NULL );
         my_bar = x.my_bar;
+        ASSERT( x.is_valid_or_zero(), "bad source for assignment" );
+        ASSERT( is_valid(), NULL );
     } 
+    bool operator==(const Foo &f) const { return my_bar == f.my_bar; }
+    bool operator<(const Foo &f) const { return my_bar < f.my_bar; }
 };
 
 class FooIterator: public std::iterator<std::input_iterator_tag,FooWithAssign> {
@@ -134,21 +146,22 @@ inline void NextSize( int& s ) {
 //! Check vector have expected size and filling
 template<typename vector_t>
 static void CheckVector( const vector_t& cv, size_t expected_size, size_t old_size ) {
+    ASSERT( cv.capacity()>=expected_size, NULL );
     ASSERT( cv.size()==expected_size, NULL );
     ASSERT( cv.empty()==(expected_size==0), NULL );
     for( int j=0; j<int(expected_size); ++j ) {
         if( cv[j].bar()!=~j )
-            std::printf("ERROR on line %d for old_size=%ld expected_size=%ld j=%d\n",__LINE__,long(old_size),long(expected_size),j);
+            REPORT("ERROR on line %d for old_size=%ld expected_size=%ld j=%d\n",__LINE__,long(old_size),long(expected_size),j);
     }
 }
 
 //! Test of assign, grow, copying with various sizes
 void TestResizeAndCopy() {
-	typedef static_counting_allocator<std::allocator<Foo>, std::size_t> allocator_t;
+    typedef static_counting_allocator<debug_allocator<Foo,std::allocator>, std::size_t> allocator_t;
     typedef tbb::concurrent_vector<Foo, allocator_t> vector_t;
     allocator_t::init_counters();
     for( int old_size=0; old_size<=128; NextSize( old_size ) ) {
-        for( int new_size=old_size; new_size<=1280; NextSize( new_size ) ) {
+        for( int new_size=0; new_size<=1280; NextSize( new_size ) ) {
             long count = FooCount;
             vector_t v;
             ASSERT( count==FooCount, NULL );
@@ -157,12 +170,12 @@ void TestResizeAndCopy() {
             for( int j=0; j<old_size/2; ++j )
                 ASSERT( v[j].state == Foo::CopyInitialized, NULL);
             v.assign(FooIterator(0), FooIterator(old_size));
-            v.grow_to_at_least(new_size);
+            v.resize(new_size, Foo(33) );
             ASSERT( count+new_size==FooCount, NULL );
             for( int j=0; j<new_size; ++j ) {
-                int expected = j<old_size ? j : Foo::initial_value_of_bar;
+                int expected = j<old_size ? j : 33;
                 if( v[j].bar()!=expected ) 
-                    std::printf("ERROR on line %d for old_size=%ld new_size=%ld v[%ld].bar()=%d != %d\n",__LINE__,long(old_size),long(new_size),long(j),v[j].bar(), expected);
+                    REPORT("ERROR on line %d for old_size=%ld new_size=%ld v[%ld].bar()=%d != %d\n",__LINE__,long(old_size),long(new_size),long(j),v[j].bar(), expected);
             }
             ASSERT( v.size()==size_t(new_size), NULL );
             for( int j=0; j<new_size; ++j ) {
@@ -186,7 +199,7 @@ void TestResizeAndCopy() {
 
 //! Test reserve, compact, capacity
 void TestCapacity() {
-    typedef static_counting_allocator<tbb::cache_aligned_allocator<Foo>, std::size_t> allocator_t;
+    typedef static_counting_allocator<debug_allocator<Foo,tbb::cache_aligned_allocator>, std::size_t> allocator_t;
     typedef tbb::concurrent_vector<Foo, allocator_t> vector_t;
     allocator_t::init_counters();
     for( size_t old_size=0; old_size<=11000; old_size=(old_size<5 ? old_size+1 : 3*old_size) ) {
@@ -199,14 +212,22 @@ void TestCapacity() {
                 ASSERT( v.capacity()>=old_size, NULL );
                 ASSERT( v.capacity()>=new_size, NULL );
                 ASSERT( v.empty(), NULL );
-                for( size_t i=0; i<2*new_size; ++i ) {
+                size_t fill_size = 2*new_size;
+                for( size_t i=0; i<fill_size; ++i ) {
                     ASSERT( size_t(FooCount)==count+i, NULL );
+#if TBB_DEPRECATED
                     size_t j = v.grow_by(1);
+#else
+                    size_t j = v.grow_by(1) - v.begin();
+#endif
                     ASSERT( j==i, NULL );
                     v[j].bar() = int(~j);
                 }
-                vector_t copy_of_v(v); // should allocate first segment with same size as for compact()
-                v.compact(); // TODO: how to check if optimization was performed correctly
+                vector_t copy_of_v(v); // should allocate first segment with same size as for shrink_to_fit()
+                if(__TBB_Log2(/*reserved size*/old_size|1) > __TBB_Log2(fill_size|1) )
+                    ASSERT( v.capacity() != copy_of_v.capacity(), NULL );
+                v.shrink_to_fit();
+                ASSERT( v.capacity() == copy_of_v.capacity(), NULL );
                 CheckVector(v, new_size*2, old_size); // check vector correctness
                 ASSERT( v==copy_of_v, NULL ); // TODO: check also segments layout equality
             }
@@ -223,7 +244,7 @@ struct AssignElement {
     void operator()( const tbb::concurrent_vector<int>::range_type& range ) const {
         for( iterator i=range.begin(); i!=range.end(); ++i ) {
             if( *i!=0 )
-                std::printf("ERROR for v[%ld]\n", long(i-base));
+                REPORT("ERROR for v[%ld]\n", long(i-base));
             *i = int(i-base);
         }
     }
@@ -236,7 +257,7 @@ struct CheckElement {
     void operator()( const tbb::concurrent_vector<int>::const_range_type& range ) const {
         for( iterator i=range.begin(); i!=range.end(); ++i )
             if( *i != int(i-base) )
-                std::printf("ERROR for v[%ld]\n", long(i-base));
+                REPORT("ERROR for v[%ld]\n", long(i-base));
     }
     CheckElement( iterator base_ ) : base(base_) {}
 };
@@ -249,21 +270,19 @@ struct CheckElement {
 void TestParallelFor( int nthread ) {
     typedef tbb::concurrent_vector<int> vector_t;
     vector_t v;
-    v.grow_to_at_least(N);  
+    v.resize(N);
     tbb::tick_count t0 = tbb::tick_count::now();
-    if( Verbose )
-        std::printf("Calling parallel_for with %ld threads\n",long(nthread));
+    REMARK("Calling parallel_for with %ld threads\n",long(nthread));
     tbb::parallel_for( v.range(10000), AssignElement(v.begin()) );
     tbb::tick_count t1 = tbb::tick_count::now();
-    const vector_t& u = v;      
+    const vector_t& u = v;
     tbb::parallel_for( u.range(10000), CheckElement(u.begin()) );
     tbb::tick_count t2 = tbb::tick_count::now();
-    if( Verbose )
-        std::printf("Time for parallel_for: assign time = %8.5f, check time = %8.5f\n",
+    REMARK("Time for parallel_for: assign time = %8.5f, check time = %8.5f\n",
                (t1-t0).seconds(),(t2-t1).seconds());
     for( long i=0; size_t(i)<v.size(); ++i )
         if( v[i]!=i )
-            std::printf("ERROR for v[%ld]\n", i);
+            REPORT("ERROR for v[%ld]\n", i);
 }
 
 template<typename Iterator1, typename Iterator2>
@@ -298,19 +317,22 @@ template<typename Vector, typename Iterator>
 void CheckConstIterator( const Vector& u, int i, const Iterator& cp ) {
     typename Vector::const_reference pref = *cp;
     if( pref.bar()!=i )
-        std::printf("ERROR for u[%ld] using const_iterator\n", long(i));
+        REPORT("ERROR for u[%ld] using const_iterator\n", long(i));
     typename Vector::difference_type delta = cp-u.begin();
     ASSERT( delta==i, NULL );
     if( u[i].bar()!=i )
-        std::printf("ERROR for u[%ld] using subscripting\n", long(i));
+        REPORT("ERROR for u[%ld] using subscripting\n", long(i));
     ASSERT( u.begin()[i].bar()==i, NULL );
 }
 
 template<typename Iterator1, typename Iterator2, typename V> 
 void CheckIteratorComparison( V& u ) {
+    V u2 = u;
     Iterator1 i = u.begin();
+
     for( int i_count=0; i_count<100; ++i_count ) {
         Iterator2 j = u.begin();
+        Iterator2 i2 = u2.begin();
         for( int j_count=0; j_count<100; ++j_count ) {
             ASSERT( (i==j)==(i_count==j_count), NULL );
             ASSERT( (i!=j)==(i_count!=j_count), NULL );
@@ -319,7 +341,10 @@ void CheckIteratorComparison( V& u ) {
             ASSERT( (i>j)==(i_count>j_count), NULL );
             ASSERT( (i<=j)==(i_count<=j_count), NULL );
             ASSERT( (i>=j)==(i_count>=j_count), NULL );
+            ASSERT( !(i==i2), NULL ); 
+            ASSERT( i!=i2, NULL ); 
             ++j;
+            ++i2;
         }
         ++i;
     }
@@ -329,10 +354,9 @@ void CheckIteratorComparison( V& u ) {
 /** Also does timing. */
 template<typename T>
 void TestSequentialFor() {
-    typedef tbb::concurrent_vector<Foo> V;
-    V v;
-    v.grow_by(N);
-    ASSERT(v.grow_by(0) == v.grow_by(0, Foo()), NULL);
+    typedef tbb::concurrent_vector<FooWithAssign> V;
+    V v(N);
+    ASSERT(v.grow_by(0) == v.grow_by(0, FooWithAssign()), NULL);
 
     // Check iterator 
     tbb::tick_count t0 = tbb::tick_count::now();
@@ -341,7 +365,7 @@ void TestSequentialFor() {
     ASSERT( !p->is_const(), NULL );
     for( int i=0; size_t(i)<v.size(); ++i, ++p ) {
         if( (*p).state!=Foo::DefaultInitialized )
-            std::printf("ERROR for v[%ld]\n", long(i));
+            REPORT("ERROR for v[%ld]\n", long(i));
         typename V::reference pref = *p;
         pref.bar() = i;
         typename V::difference_type delta = p-v.begin();
@@ -353,22 +377,26 @@ void TestSequentialFor() {
     // Check const_iterator going forwards
     const V& u = v;
     typename V::const_iterator cp = u.begin();
+    ASSERT( cp == v.cbegin(), NULL );
     ASSERT( (*cp).is_const(), NULL );
     ASSERT( cp->is_const(), NULL );
     ASSERT( *cp == v.front(), NULL);
-    for( int i=0; size_t(i)<u.size(); ++i, ++cp ) {
+    for( int i=0; size_t(i)<u.size(); ++i ) {
         CheckConstIterator(u,i,cp);
+        V::const_iterator &cpr = ++cp;
+        ASSERT( &cpr == &cp, "preincrement not returning a reference?");
     }
     tbb::tick_count t2 = tbb::tick_count::now();
-    if( Verbose )
-        std::printf("Time for serial for:  assign time = %8.5f, check time = %8.5f\n",
+    REMARK("Time for serial for:  assign time = %8.5f, check time = %8.5f\n",
                (t1-t0).seconds(),(t2-t1).seconds());
 
     // Now go backwards
     cp = u.end();
+    ASSERT( cp == v.cend(), NULL );
     for( int i=int(u.size()); i>0; ) {
         --i;
-        --cp;
+        V::const_iterator &cpr = --cp;
+        ASSERT( &cpr == &cp, "predecrement not returning a reference?");
         if( i>0 ) {
             typename V::const_iterator cp_old = cp--;
             int here = (*cp_old).bar();
@@ -387,14 +415,16 @@ void TestSequentialFor() {
         CheckConstIterator(u,int(j),cp);
         typename V::difference_type delta = i*3 % u.size();
         if( 0<=j+delta && size_t(j+delta)<u.size() ) {
-            cp += delta;
+            V::const_iterator &cpr = (cp += delta);
+            ASSERT( &cpr == &cp, "+= not returning a reference?");
             j += delta; 
         } 
         delta = i*7 % u.size();
         if( 0<=j-delta && size_t(j-delta)<u.size() ) {
-            if( i&1 ) 
-                cp -= delta;            // Test operator-=
-            else
+            if( i&1 ) { 
+                V::const_iterator &cpr = (cp -= delta);
+                ASSERT( &cpr == &cp, "-= not returning a reference?");
+            } else
                 cp = cp - delta;        // Test operator-
             j -= delta; 
         } 
@@ -404,8 +434,10 @@ void TestSequentialFor() {
         for( int j=-i; size_t(i+j)<u.size(); j=(j<50?j+1:j*5) ) {
             ASSERT( (u.begin()+i)[j].bar()==i+j, NULL );
             ASSERT( (v.begin()+i)[j].bar()==i+j, NULL );
+            ASSERT((v.cbegin()+i)[j].bar()==i+j, NULL );
             ASSERT( (i+u.begin())[j].bar()==i+j, NULL );
             ASSERT( (i+v.begin())[j].bar()==i+j, NULL );
+            ASSERT((i+v.cbegin())[j].bar()==i+j, NULL );
         }
 
     CheckIteratorComparison<typename V::iterator, typename V::iterator>(v);
@@ -415,6 +447,7 @@ void TestSequentialFor() {
 
     TestIteratorAssignment<typename V::const_iterator>( u.begin() );
     TestIteratorAssignment<typename V::const_iterator>( v.begin() );
+    TestIteratorAssignment<typename V::const_iterator>( v.cbegin() );
     TestIteratorAssignment<typename V::iterator>( v.begin() );
     // doesn't compile as expected: TestIteratorAssignment<typename V::iterator>( u.begin() );
 
@@ -434,13 +467,15 @@ void TestSequentialFor() {
     
     // Check const_reverse_iterator 
     typename V::const_reverse_iterator crp = u.rbegin();
+    ASSERT( crp == v.crbegin(), NULL );
     ASSERT( *crp == v.back(), NULL);
     for( size_t i=v.size(); i>0; --i, ++crp ) {
         typename V::const_reference cpref = *crp;
         ASSERT( size_t(cpref.bar())==i-1, NULL );
         ASSERT( crp!=u.rend(), NULL );
     }
-    ASSERT( crp==u.rend(), NULL );
+    ASSERT( crp == u.rend(), NULL );
+    ASSERT( crp == v.crend(), NULL );
 
     TestIteratorAssignment<typename V::const_reverse_iterator>( u.rbegin() );
     TestIteratorAssignment<typename V::reverse_iterator>( v.rbegin() );
@@ -456,8 +491,8 @@ void TestSequentialFor() {
 #if !defined(_WIN64) || defined(_CPPLIB_VER)
     typedef local_counting_allocator<std::allocator<int>, size_t> allocator1_t;
     typedef tbb::cache_aligned_allocator<void> allocator2_t;
-    typedef tbb::concurrent_vector<Foo, allocator1_t> V1;
-    typedef tbb::concurrent_vector<Foo, allocator2_t> V2;
+    typedef tbb::concurrent_vector<FooWithAssign, allocator1_t> V1;
+    typedef tbb::concurrent_vector<FooWithAssign, allocator2_t> V2;
     V1 v1( v ); // checking cross-allocator copying
     V2 v2( 10 ); v2 = v1; // checking cross-allocator assignment
     ASSERT( (v1 == v) && !(v2 != v), NULL);
@@ -468,53 +503,90 @@ void TestSequentialFor() {
 
 static const size_t Modulus = 7;
 
-typedef static_counting_allocator<tbb::cache_aligned_allocator<Foo> > MyAllocator;
+typedef static_counting_allocator<debug_allocator<Foo> > MyAllocator;
 typedef tbb::concurrent_vector<Foo, MyAllocator> MyVector;
 
-class GrowToAtLeast {
+template<typename MyVector>
+class GrowToAtLeast: NoAssign {
     MyVector& my_vector;
 public:
     void operator()( const tbb::blocked_range<size_t>& range ) const {
         for( size_t i=range.begin(); i!=range.end(); ++i ) {
             size_t n = my_vector.size();
-            size_t k = i % (2*n+1);
-            my_vector.grow_to_at_least(k+1);
-            ASSERT( my_vector.size()>=k+1, NULL );
+            size_t req = (i % (2*n+1))+1;
+#if TBB_DEPRECATED
+            my_vector.grow_to_at_least(req);
+#else
+            typename MyVector::iterator p = my_vector.grow_to_at_least(req);
+            if( p-my_vector.begin() < typename MyVector::difference_type(req) )
+                ASSERT( p->state == Foo::DefaultInitialized || p->state == Foo::ZeroInitialized, NULL);
+#endif
+            ASSERT( my_vector.size()>=req, NULL );
         }
     }
     GrowToAtLeast( MyVector& vector ) : my_vector(vector) {}
 };
 
 void TestConcurrentGrowToAtLeast() {
+    typedef static_counting_allocator< tbb::zero_allocator<Foo> > MyAllocator;
+    typedef tbb::concurrent_vector<Foo, MyAllocator> MyVector;
     MyAllocator::init_counters();
     MyVector v(2, Foo(), MyAllocator());
     for( size_t s=1; s<1000; s*=10 ) {
-        tbb::parallel_for( tbb::blocked_range<size_t>(0,10000*s,s), GrowToAtLeast(v) );
+        tbb::parallel_for( tbb::blocked_range<size_t>(0,10000*s,s), GrowToAtLeast<MyVector>(v), tbb::simple_partitioner() );
     }
     v.clear();
-    size_t items_allocated = v.get_allocator().items_allocated, items_freed = v.get_allocator().items_freed;
-    size_t allocations = v.get_allocator().allocations, frees = v.get_allocator().frees;
-    ASSERT( items_allocated == items_freed, NULL); ASSERT( allocations == frees, NULL);
+    ASSERT( 0 == v.get_allocator().frees, NULL);
+    v.shrink_to_fit();
+    size_t items_allocated = v.get_allocator().items_allocated,
+           items_freed = v.get_allocator().items_freed;
+    size_t allocations = v.get_allocator().allocations,
+           frees = v.get_allocator().frees;
+    ASSERT( items_allocated == items_freed, NULL);
+    ASSERT( allocations == frees, NULL);
 }
 
 //! Test concurrent invocations of method concurrent_vector::grow_by
-class GrowBy {
+template<typename MyVector>
+class GrowBy: NoAssign {
     MyVector& my_vector;
 public:
     void operator()( const tbb::blocked_range<int>& range ) const {
-        for( int i=range.begin(); i!=range.end(); ++i ) {
+        ASSERT( range.begin() < range.end(), NULL );
+#if TBB_DEPRECATED
+        for( int i=range.begin(); i!=range.end(); ++i )
+#else
+        int i = range.begin(), h = (range.end() - i) / 2;
+        typename MyVector::iterator s = my_vector.grow_by(h);
+        for( h += i; i < h; ++i, ++s )
+            s->bar() = i;
+        for(; i!=range.end(); ++i )
+#endif
+        {
             if( i&1 ) {
-                Foo& element = my_vector[my_vector.grow_by(1)]; 
+#if TBB_DEPRECATED
+                typename MyVector::reference element = my_vector[my_vector.grow_by(1)]; 
                 element.bar() = i;
+#else
+                my_vector.grow_by(1)->bar() = i;
+#endif
             } else {
-                Foo f;
+                typename MyVector::value_type f;
                 f.bar() = i;
-                size_t k;
+#if TBB_DEPRECATED
+                size_t r;
+#else
+                typename MyVector::iterator r;
+#endif
                 if( i&2 )
-                    k = my_vector.push_back( f );
+                    r = my_vector.push_back( f );
                 else
-                    k = my_vector.grow_by(1, f);
-                ASSERT( my_vector[k].bar()==i, NULL );
+                    r = my_vector.grow_by(1, f);
+#if TBB_DEPRECATED
+                ASSERT( my_vector[r].bar()==i, NULL );
+#else
+                ASSERT( r->bar()==i, NULL );
+#endif
             }
         }
     }
@@ -527,16 +599,21 @@ void TestConcurrentGrowBy( int nthread ) {
     {
         int m = 100000; MyAllocator a;
         MyVector v( a );
-        tbb::parallel_for( tbb::blocked_range<int>(0,m,1000), GrowBy(v) );
+        tbb::parallel_for( tbb::blocked_range<int>(0,m,100), GrowBy<MyVector>(v), tbb::simple_partitioner() );
         ASSERT( v.size()==size_t(m), NULL );
 
         // Verify that v is a permutation of 0..m
-        int inversions = 0;
+        int inversions = 0, def_inits = 0, copy_inits = 0;
         bool* found = new bool[m];
         memset( found, 0, m );
         for( int i=0; i<m; ++i ) {
+            if( v[i].state == Foo::DefaultInitialized ) ++def_inits;
+            else if( v[i].state == Foo::CopyInitialized ) ++copy_inits;
+            else {
+                REMARK("i: %d ", i);
+                ASSERT( false, "v[i] seems not initialized");
+            }
             int index = v[i].bar();
-            ASSERT( v[i].state == (index&1 ? Foo::DefaultInitialized : Foo::CopyInitialized), NULL);
             ASSERT( !found[index], NULL );
             found[index] = true;
             if( i>0 )
@@ -547,12 +624,18 @@ void TestConcurrentGrowBy( int nthread ) {
             ASSERT( nthread>1 || v[i].bar()==i, "sequential execution is wrong" );
         }
         delete[] found;
-        if( nthread>1 && inversions<m/10 )
-            std::printf("WARNING: not much concurrency in TestConcurrentGrowBy\n");
+        REMARK("Initialization by default constructor: %d, by copy: %d\n", def_inits, copy_inits);
+        ASSERT( def_inits >= m/2, NULL );
+        ASSERT( copy_inits >= m/4, NULL );
+        if( nthread>1 && inversions<m/20 )
+            REPORT("Warning: not much concurrency in TestConcurrentGrowBy (%d inversions)\n", inversions);
     }
-    size_t items_allocated = MyAllocator::items_allocated, items_freed = MyAllocator::items_freed;
-    size_t allocations = MyAllocator::allocations, frees = MyAllocator::frees;
-    ASSERT( items_allocated == items_freed, NULL); ASSERT( allocations == frees, NULL);
+    size_t items_allocated = MyAllocator::items_allocated,
+           items_freed = MyAllocator::items_freed;
+    size_t allocations = MyAllocator::allocations,
+           frees = MyAllocator::frees;
+    ASSERT( items_allocated == items_freed, NULL);
+    ASSERT( allocations == frees, NULL);
 }
 
 //! Test the assignment operator and swap
@@ -582,10 +665,14 @@ void TestAssign() {
             ASSERT( v.size()==size_t(src_size), NULL );
             for( int i=0; i<src_size; ++i )
                 ASSERT( v[i].bar()==i, NULL );
-            size_t items_allocated = u.get_allocator().items_allocated, items_freed = u.get_allocator().items_freed;
-            size_t allocations = u.get_allocator().allocations, frees = u.get_allocator().frees + 100;
-            ASSERT( items_allocated == items_freed, NULL); ASSERT( allocations == frees, NULL);
-            ASSERT( u.get_allocator().allocations == u.get_allocator().frees + 100, NULL);
+            ASSERT( 0 == u.get_allocator().frees, NULL);
+            u.shrink_to_fit(); // deallocate unused memory
+            size_t items_allocated = u.get_allocator().items_allocated,
+                   items_freed = u.get_allocator().items_freed;
+            size_t allocations = u.get_allocator().allocations,
+                   frees = u.get_allocator().frees + 100;
+            ASSERT( items_allocated == items_freed, NULL);
+            ASSERT( allocations == frees, NULL);
         }
     }
 }
@@ -600,8 +687,8 @@ void TestComparison() {
     var[0].assign(str[0].begin(), str[0].end());
     var[1].assign(str[0].rbegin(), str[0].rend());
     var[2].assign(var[1].rbegin(), var[1].rend()); // same as 0th
-    for (int i = 0; i < 3; i++) {
-        for (int j = 0; j < 3; j++) {
+    for (int i = 0; i < 3; ++i) {
+        for (int j = 0; j < 3; ++j) {
             ASSERT( (var[i] == var[j]) == (str[i] == str[j]), NULL );
             ASSERT( (var[i] != var[j]) == (str[i] != str[j]), NULL );
             ASSERT( (var[i] < var[j]) == (str[i] < str[j]), NULL );
@@ -640,14 +727,19 @@ public:
     void operator()( const tbb::blocked_range<Number>& r ) const {
         for( Number i=r.begin(); i!=r.end(); ++i ) { 
             if( i%2 && is_prime(i) ) {
+#if TBB_DEPRECATED
                 Primes[Primes.grow_by(1)] = i;
+#else
+                Primes.push_back( i );
+#endif
             }
         }
     }
 };
 
-static double TimeFindPrimes( int nthread ) {
-    Primes.clear();     // clear behavior has been changed since 2.0
+double TimeFindPrimes( int nthread ) {
+    Primes.clear();
+    Primes.reserve(1000000);// TODO: or compact()?
     tbb::task_scheduler_init init(nthread);
     tbb::tick_count t0 = tbb::tick_count::now();
     tbb::parallel_for( tbb::blocked_range<Number>(0,1000000,500), FindPrimes() );
@@ -655,26 +747,21 @@ static double TimeFindPrimes( int nthread ) {
     return (t1-t0).seconds();
 }
 
-static void TestFindPrimes() {
+void TestFindPrimes() {
     // Time fully subscribed run.
     double t2 = TimeFindPrimes( tbb::task_scheduler_init::automatic );
 
     // Time parallel run that is very likely oversubscribed.  
     double t128 = TimeFindPrimes(128);
 
-    if( Verbose ) 
-        std::printf("TestFindPrimes: t2==%g t128=%g k=%g\n", t2, t128, t128/t2);
+    REMARK("TestFindPrimes: t2==%g t128=%g k=%g\n", t2, t128, t128/t2);
 
     // We allow the 128-thread run a little extra time to allow for thread overhead.
     // Theoretically, following test will fail on machine with >128 processors.
     // But that situation is not going to come up in the near future,
     // and the generalization to fix the issue is not worth the trouble.
-    //
-    // [05.09.2007] Anton M has modified coefficient below from value 1.1 to 1.3 due to
-    // changes have been made in clear() behavior since 2.0 [U1] version.
-    // Originally, clear() kept segments allocated before which led to inaccurate measurment of t128.
     if( t128 > 1.3*t2 ) {
-        std::printf("Warning: grow_by is pathetically slow: t2==%g t128=%g k=%g\n", t2, t128, t128/t2);
+        REPORT("Warning: grow_by is pathetically slow: t2==%g t128=%g k=%g\n", t2, t128, t128/t2);
     } 
 }
 
@@ -698,55 +785,28 @@ void TestSort() {
 //------------------------------------------------------------------------
 // Test exceptions safety (from allocator and items constructors)
 //------------------------------------------------------------------------
-template<typename MyVector>
-class GrowthException {
-    MyVector& my_vector;
-public:
-    static volatile bool my_cancel;
-    void operator()( const tbb::blocked_range<int>& range ) const {
-        if(!my_cancel) for( int i=range.begin(); i!=range.end(); ++i ) {
-            try {
-                if( i&1 ) {
-                    Foo& element = my_vector[my_vector.grow_by(1)]; 
-                    element.bar() = i;
-                } else {
-                    Foo f;
-                    f.bar() = i;
-                    size_t k;
-                    if( i&2 )
-                        k = my_vector.push_back( f );
-                    else
-                        k = my_vector.grow_by(1, f);
-                    ASSERT( my_vector[k].bar()==i, NULL );
-                }
-            } catch(...) {
-                my_cancel = true;
-            }
-        }
-    }
-    GrowthException( MyVector& vector ) : my_vector(vector) { my_cancel = false; }
-};
-template<typename MyVector>
-volatile bool GrowthException<MyVector>::my_cancel = false;
-
+#if __TBB_EXCEPTIONS
 void TestExceptions() {
-	typedef static_counting_allocator<std::allocator<Foo>, std::size_t> allocator_t;
-    typedef tbb::concurrent_vector<Foo, allocator_t> vector_t;
+    typedef static_counting_allocator<debug_allocator<FooWithAssign>, std::size_t> allocator_t;
+    typedef tbb::concurrent_vector<FooWithAssign, allocator_t> vector_t;
 
     enum methods {
         zero_method = 0,
         ctor_copy, ctor_size, assign_nt, assign_ir, op_equ, reserve, compact, grow,
         all_methods
     };
+    ASSERT( !FooCount, NULL );
+
     try {
         vector_t src(FooIterator(0), FooIterator(N)); // original data
 
-        for(int t = 0; t < 2; t++) // exception type
-        for(int m = zero_method+1; m < all_methods; m++)
+        for(int t = 0; t < 2; ++t) // exception type
+        for(int m = zero_method+1; m < all_methods; ++m)
         {
+            ASSERT( FooCount == N, "Previous iteration miss some Foo's de-/initialization" );
             allocator_t::init_counters();
-            if(t) MaxFooCount = FooCount + N/2;
-            else allocator_t::set_limits(N/2);
+            if(t) MaxFooCount = FooCount + N/4;
+            else allocator_t::set_limits(N/4);
             vector_t victim;
             try {
                 switch(m) {
@@ -758,7 +818,7 @@ void TestExceptions() {
                     } break; // auto destruction after exception is checked by ~Foo
                 // Do not test assignment constructor due to reusing of same methods as below 
                 case assign_nt: {
-                        victim.assign(N, Foo());
+                        victim.assign(N, FooWithAssign());
                     } break;
                 case assign_ir: {
                         victim.assign(FooIterator(0), FooIterator(N));
@@ -778,13 +838,21 @@ void TestExceptions() {
                 case compact: {
                         if(t) MaxFooCount = 0; else allocator_t::set_limits(); // reset limits
                         victim.reserve(2); victim = src; // fragmented assignment
-                        if(t) MaxFooCount = 1; else allocator_t::set_limits(1, false); // block any allocation, check NULL return from allocator
-                        victim.compact(); // should start defragmenting first segment
+                        if(t) MaxFooCount = FooCount + 10; else allocator_t::set_limits(1, false); // block any allocation, check NULL return from allocator
+                        victim.shrink_to_fit(); // should start defragmenting first segment
                     } break;
                 case grow: {
-                        tbb::task_scheduler_init init;
-                        tbb::parallel_for( tbb::blocked_range<int>(0, N, 5), GrowthException<vector_t>(victim) );
-                        if(GrowthException<vector_t>::my_cancel) throw tbb::bad_last_alloc(); // parallel algorithms don't support exception passing yet
+                        tbb::task_scheduler_init init(2);
+                        if(t) MaxFooCount = FooCount + 31; // these numbers help to reproduce the live lock for versions < TBB2.2
+                        try {
+                            tbb::parallel_for( tbb::blocked_range<int>(0, N, 70), GrowBy<vector_t>(victim) );
+                        } catch(...) {
+#if TBB_USE_CAPTURED_EXCEPTION
+                            throw tbb::bad_last_alloc();
+#else
+                            throw;
+#endif
+                        }
                     } break;
                 default:;
                 }
@@ -793,6 +861,13 @@ void TestExceptions() {
                 allocator_t::set_limits(); MaxFooCount = 0;
                 size_t capacity = victim.capacity();
                 size_t size = victim.size();
+#if TBB_DEPRECATED
+                size_t req_size = victim.grow_by(0);
+#else
+                size_t req_size = victim.grow_by(0) - victim.begin();
+#endif
+                ASSERT( size <= capacity, NULL);
+                ASSERT( req_size >= size, NULL);
                 switch(m) {
                 case reserve:
                     if(t) ASSERT(false, NULL);
@@ -806,27 +881,34 @@ void TestExceptions() {
                         ASSERT(size == N, "unexpected size");
                         ASSERT(capacity >= N, "unexpected capacity");
                         int i;
-                        for(i = 1; true; i++)
+                        for(i = 1; ; ++i)
                             if(!victim[i].zero_bar()) break;
-                            else ASSERT(victim[i].bar() == (m == assign_ir)? i : Foo::initial_value_of_bar, NULL);
-                        for(; size_t(i) < size; i++) ASSERT(!victim[i].zero_bar(), NULL);
+                            else ASSERT(victim[i].bar() == (m == assign_ir)? i : initial_value_of_bar, NULL);
+                        for(; size_t(i) < size; ++i) ASSERT(!victim[i].zero_bar(), NULL);
                         ASSERT(size_t(i) == size, NULL);
                         break;
                     }
                 case grow:
                 case op_equ:
-                    if(!t) ASSERT(capacity > 0 && capacity < N, "unexpected capacity");
-                    {// not related to if(!t)
+                    if(!t) {
+                        ASSERT(capacity > 0, NULL);
+                        ASSERT(capacity < N, "unexpected capacity");
+                    }
+                    {
                         vector_t copy_of_victim(victim);
                         ASSERT(copy_of_victim.size() > 0, NULL);
-                        for(int i = 0; true; i++) {
+                        for(int i = 0; ; ++i) {
                             try {
-                                Foo &foo = victim.at(i);
-                                int bar = t? foo.zero_bar() : foo.bar();
+                                FooWithAssign &foo = victim.at(i);
+                                if( !foo.is_valid_or_zero() ) {
+                                    std::printf("i: %d size: %zd req_size: %zd  state: %d\n", i, size, req_size, foo.state);
+                                }
+                                int bar = foo.zero_bar();
                                 if(m != grow) ASSERT( bar == i || (t && bar == 0), NULL);
                                 if(size_t(i) < copy_of_victim.size()) ASSERT( copy_of_victim[i].bar() == bar, NULL);
                             } catch(std::range_error &) { // skip broken segment
-                                ASSERT( size_t(i) < size, NULL ); if(m == op_equ) break;
+                                ASSERT( size_t(i) < req_size, NULL );
+                                if(m == op_equ) break;
                             } catch(std::out_of_range &){
                                 ASSERT( i > 0, NULL ); break;
                             } catch(...) {
@@ -835,64 +917,73 @@ void TestExceptions() {
                         }
                         vector_t copy_of_victim2(10); copy_of_victim2 = victim;
                         ASSERT(copy_of_victim == copy_of_victim2, "assignment doesn't match copying");
-                        try {
-                            victim = copy_of_victim;
-                        } catch(tbb::bad_last_alloc &) { break;
-                        } catch(...) {
-                            KNOWN_ISSUE("ERROR: unrecognized exception - known compiler issue\n"); break;
+                        if(m == op_equ) {
+                            try {
+                                victim = copy_of_victim2;
+                            } catch(tbb::bad_last_alloc &) { break;
+                            } catch(...) {
+                                KNOWN_ISSUE("ERROR: unrecognized exception - known compiler issue\n"); break;
+                            }
+                            ASSERT(t, NULL);
                         }
-                        ASSERT(t, NULL);
                     } break;
                 case compact:
                     ASSERT(capacity > 0, "unexpected capacity");
-                    ASSERT(victim == src, "compact() is broken");
+                    ASSERT(victim == src, "shrink_to_fit() is broken");
                     break;
 
                 default:; // nothing to check here
                 }
-                if( Verbose ) std::printf("Exception %d: %s\t- ok ()\n", m, e.what());
+                REMARK("Exception %d: %s\t- ok\n", m, e.what());
             }
         }
     } catch(...) {
         ASSERT(false, "unexpected exception");
     }
 }
-
+#endif// __TBB_EXCEPTIONS
 //------------------------------------------------------------------------
 
-//! Test driver
+__TBB_TEST_EXPORT
 int main( int argc, char* argv[] ) {
     // Test requires at least one thread.
     MinThread = 1;
     ParseCommandLine( argc, argv );
     if( MinThread<1 ) {
-        std::printf("ERROR: MinThread=%d, but must be at least 1\n",MinThread); MinThread = 1;
+        REPORT("ERROR: MinThread=%d, but must be at least 1\n",MinThread); MinThread = 1;
     }
-    known_issue_verbose = !Verbose;
-
+#if !TBB_DEPRECATED
     TestIteratorTraits<tbb::concurrent_vector<Foo>::iterator,Foo>();
     TestIteratorTraits<tbb::concurrent_vector<Foo>::const_iterator,const Foo>();
-    TestSequentialFor<Foo> ();
+    TestSequentialFor<FooWithAssign> ();
     TestResizeAndCopy();
     TestAssign();
+#endif
     TestCapacity();
+    ASSERT( !FooCount, NULL );
     for( int nthread=MinThread; nthread<=MaxThread; ++nthread ) {
         tbb::task_scheduler_init init( nthread );
         TestParallelFor( nthread );
         TestConcurrentGrowToAtLeast();
         TestConcurrentGrowBy( nthread );
     }
+    ASSERT( !FooCount, NULL );
+#if !TBB_DEPRECATED
     TestComparison();
+#if !__TBB_FLOATING_POINT_BROKEN
     TestFindPrimes();
+#endif
     TestSort();
-#if __GLIBC__==2&&__GLIBC_MINOR__==3
-    printf("Warning: Exception safety test is skipped due to a known issue.\n");
+#if __TBB_EXCEPTIONS
+#if __TBB_EXCEPTION_HANDLING_BROKEN
+    REPORT("Warning: Exception safety test is skipped due to a known issue.\n");
 #else
     TestExceptions();
 #endif
-
-    if( Verbose ) 
-        std::printf("sizeof(concurrent_vector<int>) == %d\n", (int)sizeof(tbb::concurrent_vector<int>));
-    std::printf("done\n");
+#endif//__TBB_EXCEPTIONS
+#endif//!TBB_DEPRECATED
+    ASSERT( !FooCount, NULL );
+    REMARK("sizeof(concurrent_vector<int>) == %d\n", (int)sizeof(tbb::concurrent_vector<int>));
+    REPORT("done\n");
     return 0;
 }
