@@ -31,11 +31,13 @@ const int MByte = 1048576; //1MB
 /* _WIN32_WINNT should be defined at the very beginning, 
    because other headers might include <windows.h>
 */
+
 #if _WIN32 || _WIN64
 #undef _WIN32_WINNT
 #define _WIN32_WINNT 0x0500
 #include <windows.h>
 #include <stdio.h>
+#include "harness_report.h"
 
 void limitMem( int limit )
 {
@@ -46,17 +48,17 @@ void limitMem( int limit )
     jobInfo.ProcessMemoryLimit = limit? limit*MByte : 2*1024LL*MByte;
     if (NULL == hJob) {
         if (NULL == (hJob = CreateJobObject(NULL, NULL))) {
-            printf("Can't assign create job object: %d\n", GetLastError());
+            REPORT("Can't assign create job object: %ld\n", GetLastError());
             exit(1);
         }
         if (0 == AssignProcessToJobObject(hJob, GetCurrentProcess())) {
-            printf("Can't assign process to job object: %d\n", GetLastError());
+            REPORT("Can't assign process to job object: %ld\n", GetLastError());
             exit(1);
         }
     }
     if (0 == SetInformationJobObject(hJob, JobObjectExtendedLimitInformation, 
                                      &jobInfo, sizeof(jobInfo))) {
-        printf("Can't set limits: %d\n", GetLastError());
+        REPORT("Can't set limits: %ld\n", GetLastError());
         exit(1);
     }
 }
@@ -65,6 +67,8 @@ void limitMem( int limit )
 #include <stdlib.h>
 #include <stdio.h>
 #include <errno.h>
+#include <sys/types.h>  // uint64_t on FreeBSD, needed for rlim_t
+#include "harness_report.h"
 
 void limitMem( int limit )
 {
@@ -73,7 +77,7 @@ void limitMem( int limit )
     rlim.rlim_max = (rlim_t)RLIM_INFINITY;
     int ret = setrlimit(RLIMIT_AS,&rlim);
     if (0 != ret) {
-        printf("Can't set limits: errno %d\n", errno);
+        REPORT("Can't set limits: errno %d\n", errno);
         exit(1);
     }
 }
@@ -84,8 +88,10 @@ void limitMem( int limit )
 #include <vector>
 #define __TBB_NO_IMPLICIT_LINKAGE 1
 #include "tbb/scalable_allocator.h"
+#include "tbb/tbb_machine.h"
 
 #include "harness.h"
+#include "harness_barrier.h"
 #if __linux__
 #include <stdint.h> // uintptr_t
 #endif
@@ -95,7 +101,7 @@ void limitMem( int limit )
 
 const size_t COUNT_ELEM_CALLOC = 2;
 const int COUNT_TESTS = 1000;
-const int COUNT_ELEM = 50000;
+const int COUNT_ELEM = 25000;
 const size_t MAX_SIZE = 1000;
 const int COUNTEXPERIMENT = 10000;
 
@@ -142,6 +148,28 @@ static bool perProcessLimits = true;
 
 const size_t POWERS_OF_2 = 20;
 
+#if __linux__  && __ia64__
+/* Can't use Intel compiler intrinsic due to internal error reported by
+   10.1 compiler */
+pthread_mutex_t counter_mutex = PTHREAD_MUTEX_INITIALIZER;
+
+int32_t __TBB_machine_fetchadd4__TBB_full_fence (volatile void *ptr, int32_t value)
+{
+    pthread_mutex_lock(&counter_mutex);
+    int32_t result = *(int32_t*)ptr;
+    *(int32_t*)ptr = result + value;
+    pthread_mutex_unlock(&counter_mutex);
+    return result;
+}
+
+void __TBB_machine_pause(int32_t /*delay*/) {}
+
+#elif (_WIN32||_WIN64) && defined(_M_AMD64)
+
+void __TBB_machine_pause(__int32 /*delay*/ ) {}
+
+#endif
+
 struct MemStruct
 {
     void* Pointer;
@@ -154,21 +182,20 @@ struct MemStruct
 class CMemTest: NoAssign
 {
     UINT CountErrors;
-    int total_threads;
     bool FullLog;
+    Harness::SpinBarrier *limitBarrier;
     static bool firstTime;
 
 public:
-    CMemTest(int total_threads,
-             bool isVerbose=false) :
-        CountErrors(0), total_threads(total_threads)
+    CMemTest(Harness::SpinBarrier *limitBarrier, bool isVerbose=false) :
+        CountErrors(0), limitBarrier(limitBarrier)
         {
             srand((UINT)time(NULL));
             FullLog=isVerbose;
             rand();
         }
     void InvariantDataRealloc(bool aligned); //realloc does not change data
-    void NULLReturn(UINT MinSize, UINT MaxSize); // NULL pointer + check errno
+    void NULLReturn(UINT MinSize, UINT MaxSize, int total_threads); // NULL pointer + check errno
     void UniquePointer(); // unique pointer - check with padding
     void AddrArifm(); // unique pointer - check with pointer arithmetic
     bool ShouldReportError();
@@ -179,6 +206,15 @@ public:
     ~CMemTest() {}
 };
 
+class Limit {
+    int limit;
+public:
+    Limit(int limit) : limit(limit) {}
+    void operator() () const {
+        limitMem(limit);
+    }
+};
+
 int argC;
 char** argV;
 
@@ -186,8 +222,8 @@ struct RoundRobin: NoAssign {
     const long number_of_threads;
     mutable CMemTest test;
 
-    RoundRobin( long p, bool verbose ) :
-        number_of_threads(p), test(p, verbose) {}
+    RoundRobin( long p, Harness::SpinBarrier *limitBarrier, bool verbose ) :
+        number_of_threads(p), test(limitBarrier, verbose) {}
     void operator()( int /*id*/ ) const 
         {
             test.RunAllTests(number_of_threads);
@@ -249,6 +285,57 @@ void ReallocParam()
         Trealloc(bufs[j], 0);
 }
 
+__TBB_TEST_EXPORT
+int main(int argc, char* argv[]) {
+    argC=argc;
+    argV=argv;
+    MaxThread = MinThread = 1;
+    Tmalloc=scalable_malloc;
+    Trealloc=scalable_realloc;
+    Tcalloc=scalable_calloc;
+    Tfree=scalable_free;
+    Rposix_memalign=scalable_posix_memalign;
+    Raligned_malloc=scalable_aligned_malloc;
+    Raligned_realloc=scalable_aligned_realloc;
+    Taligned_free=scalable_aligned_free;
+
+    // check if we were called to test standard behavior
+    for (int i=1; i< argc; i++) {
+        if (strcmp((char*)*(argv+i),"-s")==0)
+        {
+            setSystemAllocs();
+            argC--;
+            break;
+        }
+    }
+
+    ParseCommandLine( argC, argV );
+#if __linux__
+    /* According to man pthreads 
+       "NPTL threads do not share resource limits (fixed in kernel 2.6.10)".
+       Use per-threads limits for affected systems.
+     */
+    if ( LinuxKernelVersion() < 2*1000000 + 6*1000 + 10)
+        perProcessLimits = false;
+#endif    
+    //-------------------------------------
+#if __APPLE__
+    /* Skip due to lack of memory limit enforcing under Mac OS X. */
+#else
+    limitMem(200);
+    ReallocParam();
+    limitMem(0);
+#endif
+    for( int p=MaxThread; p>=MinThread; --p ) {
+        REMARK("testing with %d threads\n", p );
+        Harness::SpinBarrier *barrier = new Harness::SpinBarrier(p);
+        NativeParallelFor( p, RoundRobin(p, barrier, Verbose) );
+        delete barrier;
+    }
+    if( !error_occurred ) REPORT("done\n");
+    return 0;
+}
+
 struct TestStruct
 {
     DWORD field1:2;
@@ -303,7 +390,7 @@ void CMemTest::InvariantDataRealloc(bool aligned)
 {
     size_t size, sizeMin;
     CountErrors=0;
-    if (FullLog) printf("\nInvariant data by realloc....");
+    if (FullLog) REPORT("\nInvariant data by realloc....");
     UCHAR* pchar;
     sizeMin=size=rand()%MAX_SIZE+10;
     pchar = aligned?
@@ -329,8 +416,8 @@ void CMemTest::InvariantDataRealloc(bool aligned)
                 CountErrors++;
                 if (ShouldReportError())
                 {
-                    printf("stand '%c', must stand '%c'\n",pchar[k],(UCHAR)k%255+1);
-                    printf("error: data changed (at %llu, SizeMin=%llu)\n",
+                    REPORT("stand '%c', must stand '%c'\n",pchar[k],(UCHAR)k%255+1);
+                    REPORT("error: data changed (at %llu, SizeMin=%llu)\n",
                            (long long unsigned)k,(long long unsigned)sizeMin);
                 }
             }
@@ -339,141 +426,91 @@ void CMemTest::InvariantDataRealloc(bool aligned)
         Taligned_realloc(pchar,0,choose_random_alignment());
     else
         Trealloc(pchar,0);
-    if (CountErrors) printf("%s\n",strError);
-    else if (FullLog) printf("%s\n",strOk);
+    if (CountErrors) REPORT("%s\n",strError);
+    else if (FullLog) REPORT("%s\n",strOk);
     error_occurred |= ( CountErrors>0 ) ;
-    //printf("end check\n");
+    //REPORT("end check\n");
+}
+
+struct PtrSize {
+    void  *ptr;
+    size_t size;
+};
+
+static int cmpAddrs(const void *p1, const void *p2)
+{
+    const PtrSize *a = (const PtrSize *)p1;
+    const PtrSize *b = (const PtrSize *)p2;
+
+    return a->ptr < b->ptr ? -1 : ( a->ptr == b->ptr ? 0 : 1);
 }
 
 void CMemTest::AddrArifm()
 {
-    int* MasPointer[COUNT_ELEM];
-    size_t *MasCountElem = (size_t*)Tmalloc(COUNT_ELEM*sizeof(size_t));
-    size_t count;
-    int* tmpAddr;
-    int j;
-    UINT CountZero=0;
-    CountErrors=0;
-    if (FullLog) printf("\nUnique pointer using Address arithmetics\n");
-    if (FullLog) printf("malloc....");
-    ASSERT(MasCountElem, NULL);
-    for (int i=0; i<COUNT_ELEM; i++)
-    {
-        count=rand()%MAX_SIZE;
-        if (count == 0)
-            CountZero++;
-        tmpAddr=(int*)Tmalloc(count*sizeof(int));
-        for (j=0; j<i; j++) // find a place for the new address
-        {
-            if (*(MasPointer+j)>tmpAddr) break;
-        }
-        for (int k=i; k>j; k--)
-        {
-            MasPointer[k]=MasPointer[k-1];
-            MasCountElem[k]=MasCountElem[k-1];
-        }
-        MasPointer[j]=tmpAddr;
-        MasCountElem[j]=count*sizeof(int);/**/
-    }
-    if (FullLog) printf("Count zero: %d\n",CountZero);
-    for (int i=0; i<COUNT_ELEM-1; i++)
-    {
-        if (NULL!=MasPointer[i] && NULL!=MasPointer[i+1]
-            && (uintptr_t)MasPointer[i]+MasCountElem[i] > (uintptr_t)MasPointer[i+1])
-        {
-            CountErrors++;
-            if (ShouldReportError()) printf("malloc: intersection detect at 0x%p between %llu element(int) and 0x%p\n",
-                                            (MasPointer+i),(long long unsigned)MasCountElem[i],(MasPointer+i+1));
-        }
-    }
-    if (CountErrors) printf("%s\n",strError);
-    else if (FullLog) printf("%s\n",strOk);
-    error_occurred |= ( CountErrors>0 ) ;
-    //----------------------------------------------------------------
-    CountErrors=0;
-    if (FullLog) printf("realloc....");
-    for (int i=0; i<COUNT_ELEM; i++)
-    {
-        count=MasCountElem[i]*2;
-        if (count == 0)
-            CountZero++;
-        tmpAddr=(int*)Trealloc(MasPointer[i],count);
-        if (NULL==tmpAddr && 0!=count)
-            Tfree(MasPointer[i]);
-        for (j=0; j<i; j++) // find a place for the new address
-        {
-            if (*(MasPointer+j)>tmpAddr) break;
-        }
-        for (int k=i; k>j; k--)
-        {
-            MasPointer[k]=MasPointer[k-1];
-            MasCountElem[k]=MasCountElem[k-1];
-        }
-        MasPointer[j]=tmpAddr;
-        MasCountElem[j]=count;//*sizeof(int);/**/
-    }
-    if (FullLog) printf("Count zero: %d\n",CountZero);
+    PtrSize *arr = (PtrSize*)Tmalloc(COUNT_ELEM*sizeof(PtrSize));
 
-    // now we have a sorted array of pointers
-    for (int i=0; i<COUNT_ELEM-1; i++)
-    {
-        if (NULL!=MasPointer[i] && NULL!=MasPointer[i+1]
-            && (uintptr_t)MasPointer[i]+MasCountElem[i] > (uintptr_t)MasPointer[i+1])
-        {
-            CountErrors++;
-            if (ShouldReportError()) printf("realloc: intersection detect at 0x%p between %llu element(int) and 0x%p\n",
-                                            (MasPointer+i),(long long unsigned)MasCountElem[i],(MasPointer+i+1));
-        }
-    }
-    if (CountErrors) printf("%s\n",strError);
-    else if (FullLog) printf("%s\n",strOk);
-    error_occurred |= ( CountErrors>0 ) ;
+    if (FullLog) REPORT("\nUnique pointer using Address arithmetics\n");
+    if (FullLog) REPORT("malloc....");
+    ASSERT(arr, NULL);
     for (int i=0; i<COUNT_ELEM; i++)
     {
-        Tfree(MasPointer[i]);
+        arr[i].size=rand()%MAX_SIZE;
+        arr[i].ptr=Tmalloc(arr[i].size);
+    }
+    qsort(arr, COUNT_ELEM, sizeof(PtrSize), cmpAddrs);
+
+    for (int i=0; i<COUNT_ELEM-1; i++)
+    {
+        if (NULL!=arr[i].ptr && NULL!=arr[i+1].ptr)
+            ASSERT((uintptr_t)arr[i].ptr+arr[i].size <= (uintptr_t)arr[i+1].ptr,
+                   "intersection detected");
+    }
+    //----------------------------------------------------------------
+    if (FullLog) REPORT("realloc....");
+    for (int i=0; i<COUNT_ELEM; i++)
+    {
+        size_t count=arr[i].size*2;
+        void *tmpAddr=Trealloc(arr[i].ptr,count);
+        if (NULL!=tmpAddr) {
+            arr[i].ptr = tmpAddr;
+            arr[i].size = count;
+        } else if (count==0) { // becasue realloc(..., 0) works as free
+            arr[i].ptr = NULL;
+            arr[i].size = 0;
+        }
+    }
+    qsort(arr, COUNT_ELEM, sizeof(PtrSize), cmpAddrs);
+
+    for (int i=0; i<COUNT_ELEM-1; i++)
+    {
+        if (NULL!=arr[i].ptr && NULL!=arr[i+1].ptr)
+            ASSERT((uintptr_t)arr[i].ptr+arr[i].size <= (uintptr_t)arr[i+1].ptr,
+                   "intersection detected");
+    }
+    for (int i=0; i<COUNT_ELEM; i++)
+    {
+        Tfree(arr[i].ptr);
     }
     //-------------------------------------------
-    CountErrors=0;
-    if (FullLog) printf("calloc....");
+    if (FullLog) REPORT("calloc....");
     for (int i=0; i<COUNT_ELEM; i++)
     {
-        count=rand()%MAX_SIZE;
-        if (count == 0)
-            CountZero++;
-        tmpAddr=(int*)Tcalloc(count*sizeof(int),2);
-        for (j=0; j<i; j++) // find a place for the new address
-        {
-            if (*(MasPointer+j)>tmpAddr) break;
-        }
-        for (int k=i; k>j; k--)
-        {
-            MasPointer[k]=MasPointer[k-1];
-            MasCountElem[k]=MasCountElem[k-1];
-        }
-        MasPointer[j]=tmpAddr;
-        MasCountElem[j]=count*sizeof(int)*2;/**/
+        arr[i].size=rand()%MAX_SIZE;
+        arr[i].ptr=Tcalloc(arr[i].size,1);
     }
-    if (FullLog) printf("Count zero: %d\n",CountZero);
+    qsort(arr, COUNT_ELEM, sizeof(PtrSize), cmpAddrs);
 
-    // now we have a sorted array of pointers
     for (int i=0; i<COUNT_ELEM-1; i++)
     {
-        if (NULL!=MasPointer[i] && NULL!=MasPointer[i+1]
-            && (uintptr_t)MasPointer[i]+MasCountElem[i] > (uintptr_t)MasPointer[i+1])
-        {
-            CountErrors++;
-            if (ShouldReportError()) printf("calloc: intersection detect at 0x%p between %llu element(int) and 0x%p\n",
-                                            (MasPointer+i),(long long unsigned)MasCountElem[i],(MasPointer+i+1));
-        }
+        if (NULL!=arr[i].ptr && NULL!=arr[i+1].ptr)
+            ASSERT((uintptr_t)arr[i].ptr+arr[i].size <= (uintptr_t)arr[i+1].ptr,
+                   "intersection detected");
     }
-    if (CountErrors) printf("%s\n",strError);
-    else if (FullLog) printf("%s\n",strOk);
-    error_occurred |= ( CountErrors>0 ) ;
     for (int i=0; i<COUNT_ELEM; i++)
     {
-        Tfree(MasPointer[i]);
+        Tfree(arr[i].ptr);
     }
-    Tfree(MasCountElem);
+    Tfree(arr);
 }
 
 void CMemTest::Zerofilling()
@@ -481,7 +518,7 @@ void CMemTest::Zerofilling()
     TestStruct* TSMas;
     size_t CountElement;
     CountErrors=0;
-    if (FullLog) printf("\nzeroings elements of array....");
+    if (FullLog) REPORT("\nzeroings elements of array....");
     //test struct
     for (int i=0; i<COUNTEXPERIMENT; i++)
     {
@@ -491,56 +528,79 @@ void CMemTest::Zerofilling()
             continue;
         for (size_t j=0; j<CountElement; j++)
         {
-            if (!TSMas->IzZero())
+            if (!(TSMas+j)->IzZero())
             {
                 CountErrors++;
-                if (ShouldReportError()) printf("detect nonzero element at TestStruct\n");
+                if (ShouldReportError()) REPORT("detect nonzero element at TestStruct\n");
             }
         }
         Tfree(TSMas);
     }
-    if (CountErrors) printf("%s\n",strError);
-    else if (FullLog) printf("%s\n",strOk);
+    if (CountErrors) REPORT("%s\n",strError);
+    else if (FullLog) REPORT("%s\n",strOk);
     error_occurred |= ( CountErrors>0 ) ;
 }
 
-// As several threads concurrently trying to push to memory limits, adding to 
-// vectors may have intermittent failures.  
-void reliablePushBack(std::vector<MemStruct> *vec, const MemStruct &mStruct)
+#if !__APPLE__
+void CMemTest::NULLReturn(UINT MinSize, UINT MaxSize, int total_threads)
 {
-    for (int i=0; i<10000; i++) {
-        try {
-            vec->push_back(mStruct);
-        } catch(std::bad_alloc) {
-            continue;
-        }
-        return;
-    }
-    ASSERT(0, "Unable to get free memory.");
-}
+    // find size to guarantee getting NULL for 1024 B allocations
+    const int MAXNUM_1024 = (200+50)*1024;
 
-void CMemTest::NULLReturn(UINT MinSize, UINT MaxSize)
-{
     std::vector<MemStruct> PointerList;
     void *tmp;
     CountErrors=0;
-    int CountNULL;
-    if (FullLog) printf("\nNULL return & check errno:\n");
+    int CountNULL, num_1024;
+    if (FullLog) REPORT("\nNULL return & check errno:\n");
     UINT Size;
+    Limit limit_200M(200*total_threads), no_limit(0);
+    void **buf_1024 = (void**)Tmalloc(MAXNUM_1024*sizeof(void*));
+
+    ASSERT(buf_1024, NULL);
+    /* We must have space for pointers when memory limit is hit. 
+       Reserve enough for the worst case. 
+    */
+    PointerList.reserve(200*MByte/MinSize);
+
+    /* There is a bug in the specific verion of GLIBC (2.5-12) shipped 
+       with RHEL5 that leads to erroneous working of the test 
+       on Intel64 and IPF systems when setrlimit-related part is enabled.
+       Switching to GLIBC 2.5-18 from RHEL5.1 resolved the issue.
+     */
+    if (perProcessLimits)
+        limitBarrier->wait(limit_200M);
+    else
+        limitMem(200);
+
+    /* regression test against the bug in allocator when it dereference NULL 
+       while lack of memory 
+    */
+    for (num_1024=0; num_1024<MAXNUM_1024; num_1024++) {
+        buf_1024[num_1024] = Tcalloc(1024, 1);
+        if (! buf_1024[num_1024]) {
+            ASSERT(errno == ENOMEM, NULL);
+            break;
+        }
+    }
+    for (int i=0; i<num_1024; i++)
+        Tfree(buf_1024[i]);
+    Tfree(buf_1024);
+
     do {
         Size=rand()%(MaxSize-MinSize)+MinSize;
         tmp=Tmalloc(Size);
         if (tmp != NULL)
         {
             memset(tmp, 0, Size);
-            reliablePushBack(&PointerList, MemStruct(tmp, Size));
+            PointerList.push_back(MemStruct(tmp, Size));
         }
     } while(tmp != NULL);
-    if (FullLog) printf("\n");
+    ASSERT(errno == ENOMEM, NULL);
+    if (FullLog) REPORT("\n");
 
     // preparation complete, now running tests
     // malloc
-    if (FullLog) printf("malloc....");
+    if (FullLog) REPORT("malloc....");
     CountNULL = 0;
     while (CountNULL==0)
         for (int j=0; j<COUNT_TESTS; j++)
@@ -553,7 +613,7 @@ void CMemTest::NULLReturn(UINT MinSize, UINT MaxSize)
                 CountNULL++;
                 if (errno != ENOMEM) {
                     CountErrors++;
-                    if (ShouldReportError()) printf("NULL returned, error: errno (%d) != ENOMEM\n", errno);
+                    if (ShouldReportError()) REPORT("NULL returned, error: errno (%d) != ENOMEM\n", errno);
                 }
             }
             else
@@ -566,20 +626,20 @@ void CMemTest::NULLReturn(UINT MinSize, UINT MaxSize)
 #endif /* __linux__ */
                 if (errno != ENOMEM+j+1 && !known_issue) {
                     CountErrors++;
-                    if (ShouldReportError()) printf("error: errno changed to %d though valid pointer was returned\n", errno);
+                    if (ShouldReportError()) REPORT("error: errno changed to %d though valid pointer was returned\n", errno);
                 }      
                 memset(tmp, 0, Size);
-                reliablePushBack(&PointerList, MemStruct(tmp, Size));
+                PointerList.push_back(MemStruct(tmp, Size));
             }
         }
-    if (FullLog) printf("end malloc\n");
-    if (CountErrors) printf("%s\n",strError);
-    else if (FullLog) printf("%s\n",strOk);
+    if (FullLog) REPORT("end malloc\n");
+    if (CountErrors) REPORT("%s\n",strError);
+    else if (FullLog) REPORT("%s\n",strOk);
     error_occurred |= ( CountErrors>0 ) ;
 
     CountErrors=0;
     //calloc
-    if (FullLog) printf("calloc....");
+    if (FullLog) REPORT("calloc....");
     CountNULL = 0;
     while (CountNULL==0)
         for (int j=0; j<COUNT_TESTS; j++)
@@ -592,7 +652,7 @@ void CMemTest::NULLReturn(UINT MinSize, UINT MaxSize)
                 CountNULL++;
                 if (errno != ENOMEM) {
                     CountErrors++;
-                    if (ShouldReportError()) printf("NULL returned, error: errno(%d) != ENOMEM\n", errno);
+                    if (ShouldReportError()) REPORT("NULL returned, error: errno(%d) != ENOMEM\n", errno);
                 }
             }
             else
@@ -605,17 +665,17 @@ void CMemTest::NULLReturn(UINT MinSize, UINT MaxSize)
 #endif /* __linux__ */
                 if (errno != ENOMEM+j+1 && !known_issue) {
                     CountErrors++;
-                    if (ShouldReportError()) printf("error: errno changed to %d though valid pointer was returned\n", errno);
+                    if (ShouldReportError()) REPORT("error: errno changed to %d though valid pointer was returned\n", errno);
                 }      
-                reliablePushBack(&PointerList, MemStruct(tmp, Size));
+                PointerList.push_back(MemStruct(tmp, Size));
             }
         }
-    if (FullLog) printf("end calloc\n");
-    if (CountErrors) printf("%s\n",strError);
-    else if (FullLog) printf("%s\n",strOk);
+    if (FullLog) REPORT("end calloc\n");
+    if (CountErrors) REPORT("%s\n",strError);
+    else if (FullLog) REPORT("%s\n",strOk);
     error_occurred |= ( CountErrors>0 ) ;
     CountErrors=0;
-    if (FullLog) printf("realloc....");
+    if (FullLog) REPORT("realloc....");
     CountNULL = 0;
     if (PointerList.size() > 0)
         while (CountNULL==0)
@@ -631,7 +691,7 @@ void CMemTest::NULLReturn(UINT MinSize, UINT MaxSize)
 #endif /* __linux__ */
                     if (errno != 0 && !known_issue) {
                         CountErrors++;
-                        if (ShouldReportError()) printf("valid pointer returned, error: errno not kept\n");
+                        if (ShouldReportError()) REPORT("valid pointer returned, error: errno not kept\n");
                     }      
                     PointerList[i].Size *= 2;
                 }
@@ -643,7 +703,7 @@ void CMemTest::NULLReturn(UINT MinSize, UINT MaxSize)
 #endif /* __linux__ */
                     if (errno != 0 && !known_issue) {
                         CountErrors++;
-                        if (ShouldReportError()) printf("valid pointer returned, error: errno not kept\n");
+                        if (ShouldReportError()) REPORT("valid pointer returned, error: errno not kept\n");
                     }      
                     PointerList[i].Pointer = tmp;
                     PointerList[i].Size *= 2;
@@ -654,7 +714,7 @@ void CMemTest::NULLReturn(UINT MinSize, UINT MaxSize)
                     if (errno != ENOMEM)
                     {
                         CountErrors++;
-                        if (ShouldReportError()) printf("NULL returned, error: errno(%d) != ENOMEM\n", errno);
+                        if (ShouldReportError()) REPORT("NULL returned, error: errno(%d) != ENOMEM\n", errno);
                     }
                     // check data integrity
                     BYTE *zer=(BYTE*)PointerList[i].Pointer;
@@ -662,28 +722,32 @@ void CMemTest::NULLReturn(UINT MinSize, UINT MaxSize)
                         if (zer[k] != 0)
                         {
                             CountErrors++;
-                            if (ShouldReportError()) printf("NULL returned, error: data changed\n");
+                            if (ShouldReportError()) REPORT("NULL returned, error: data changed\n");
                         }
                 }
             }
-    if (FullLog) printf("realloc end\n");
-    if (CountErrors) printf("%s\n",strError);
-    else if (FullLog) printf("%s\n",strOk);
+    if (FullLog) REPORT("realloc end\n");
+    if (CountErrors) REPORT("%s\n",strError);
+    else if (FullLog) REPORT("%s\n",strOk);
     error_occurred |= ( CountErrors>0 ) ;
     for (UINT i=0; i<PointerList.size(); i++)
     {
         Tfree(PointerList[i].Pointer);
     }
+    if (perProcessLimits)
+        limitBarrier->wait(no_limit);
+    else
+        limitMem(0);
 }
-
+#endif /* #if __APPLE__ */
 
 void CMemTest::UniquePointer()
 {
     CountErrors=0;
-    int* MasPointer[COUNT_ELEM];
+    int **MasPointer = (int **)Tmalloc(sizeof(int*)*COUNT_ELEM);
     size_t *MasCountElem = (size_t*)Tmalloc(sizeof(size_t)*COUNT_ELEM);
-    if (FullLog) printf("\nUnique pointer using 0\n");
-    ASSERT(MasCountElem, NULL);
+    if (FullLog) REPORT("\nUnique pointer using 0\n");
+    ASSERT(MasCountElem && MasPointer, NULL);
     //
     //-------------------------------------------------------
     //malloc
@@ -696,7 +760,7 @@ void CMemTest::UniquePointer()
         for (UINT j=0; j<MasCountElem[i]; j++)
             *(MasPointer[i]+j)=0;
     }
-    if (FullLog) printf("malloc....");
+    if (FullLog) REPORT("malloc....");
     for (UINT i=0; i<COUNT_ELEM-1; i++)
     {
         for (UINT j=0; j<MasCountElem[i]; j++)
@@ -704,13 +768,13 @@ void CMemTest::UniquePointer()
             if (*(*(MasPointer+i)+j)!=0)
             {
                 CountErrors++;
-                if (ShouldReportError()) printf("error, detect 1 with 0x%p\n",(*(MasPointer+i)+j));
+                if (ShouldReportError()) REPORT("error, detect 1 with 0x%p\n",(*(MasPointer+i)+j));
             }
             *(*(MasPointer+i)+j)+=1;
         }
     }
-    if (CountErrors) printf("%s\n",strError);
-    else if (FullLog) printf("%s\n",strOk);
+    if (CountErrors) REPORT("%s\n",strError);
+    else if (FullLog) REPORT("%s\n",strOk);
     error_occurred |= ( CountErrors>0 ) ;
     //----------------------------------------------------------
     //calloc
@@ -723,7 +787,7 @@ void CMemTest::UniquePointer()
         if (NULL == MasPointer[i])
             MasCountElem[i]=0;
     }
-    if (FullLog) printf("calloc....");
+    if (FullLog) REPORT("calloc....");
     for (int i=0; i<COUNT_ELEM-1; i++)
     {
         for (UINT j=0; j<*(MasCountElem+i); j++)
@@ -731,13 +795,13 @@ void CMemTest::UniquePointer()
             if (*(*(MasPointer+i)+j)!=0)
             {
                 CountErrors++;
-                if (ShouldReportError()) printf("error, detect 1 with 0x%p\n",(*(MasPointer+i)+j));
+                if (ShouldReportError()) REPORT("error, detect 1 with 0x%p\n",(*(MasPointer+i)+j));
             }
             *(*(MasPointer+i)+j)+=1;
         }
     }
-    if (CountErrors) printf("%s\n",strError);
-    else if (FullLog) printf("%s\n",strOk);
+    if (CountErrors) REPORT("%s\n",strError);
+    else if (FullLog) REPORT("%s\n",strOk);
     error_occurred |= ( CountErrors>0 ) ;
     //---------------------------------------------------------
     //realloc
@@ -752,7 +816,7 @@ void CMemTest::UniquePointer()
         for (UINT j=0; j<MasCountElem[i]; j++)
             *(*(MasPointer+i)+j)=0;
     }
-    if (FullLog) printf("realloc....");
+    if (FullLog) REPORT("realloc....");
     for (int i=0; i<COUNT_ELEM-1; i++)
     {
         for (UINT j=0; j<*(MasCountElem+i); j++)
@@ -764,12 +828,13 @@ void CMemTest::UniquePointer()
             *(*(MasPointer+i)+j)+=1;
         }
     }
-    if (CountErrors) printf("%s\n",strError);
-    else if (FullLog) printf("%s\n",strOk);
+    if (CountErrors) REPORT("%s\n",strError);
+    else if (FullLog) REPORT("%s\n",strOk);
     error_occurred |= ( CountErrors>0 ) ;
     for (int i=0; i<COUNT_ELEM; i++)
         Tfree(MasPointer[i]);
     Tfree(MasCountElem);
+    Tfree(MasPointer);
 }
 
 bool CMemTest::ShouldReportError()
@@ -787,7 +852,7 @@ bool CMemTest::ShouldReportError()
 void CMemTest::Free_NULL()
 {
     CountErrors=0;
-    if (FullLog) printf("\ncall free with parameter NULL....");
+    if (FullLog) REPORT("\ncall free with parameter NULL....");
     errno = 0;
     for (int i=0; i<COUNTEXPERIMENT; i++)
     {
@@ -795,11 +860,11 @@ void CMemTest::Free_NULL()
         if (errno != 0)
         {
             CountErrors++;
-            if (ShouldReportError()) printf("error is found by a call free with parameter NULL\n");
+            if (ShouldReportError()) REPORT("error is found by a call free with parameter NULL\n");
         }
     }
-    if (CountErrors) printf("%s\n",strError);
-    else if (FullLog) printf("%s\n",strOk);
+    if (CountErrors) REPORT("%s\n",strError);
+    else if (FullLog) REPORT("%s\n",strOk);
     error_occurred |= ( CountErrors>0 ) ;
 }
 
@@ -874,78 +939,23 @@ void CMemTest::TestAlignedParameters()
     }
 }
 
-void CMemTest::RunAllTests(int /*total_threads*/)
+void CMemTest::RunAllTests(int total_threads)
 {
+
     Zerofilling();
     Free_NULL();
     InvariantDataRealloc(/*aligned=*/false);
     if (Raligned_realloc)
         InvariantDataRealloc(/*aligned=*/true);
     TestAlignedParameters();
-    if (FullLog) printf("All tests ended\nclearing memory....");
-}
-
-int main(int argc, char* argv[])
-{
-    argC=argc;
-    argV=argv;
-    MaxThread = MinThread = 1;
-    Tmalloc=scalable_malloc;
-    Trealloc=scalable_realloc;
-    Tcalloc=scalable_calloc;
-    Tfree=scalable_free;
-    Rposix_memalign=scalable_posix_memalign;
-    Raligned_malloc=scalable_aligned_malloc;
-    Raligned_realloc=scalable_aligned_realloc;
-    Taligned_free=scalable_aligned_free;
-
-    // check if we were called to test standard behavior
-    for (int i=1; i< argc; i++) {
-        if (strcmp((char*)*(argv+i),"-s")==0)
-        {
-            setSystemAllocs();
-            argC--;
-            break;
-        }
-    }
-
-    ParseCommandLine( argC, argV );
-
-    CMemTest test(1, Verbose);
-    // Tests with memory limit also run in serial only
 #if __APPLE__
-    /* Skip due to lack of memory limit enforcing under Mac OS X. */
+    REPORT("Warning: skipping some tests (known issue on Mac OS* X)\n");
 #else
-    limitMem(200);
-    ReallocParam(); /* this test should be first to have enough unfragmented memory */
-    test.NULLReturn(1*MByte,100*MByte);
-    limitMem(0);
+    UniquePointer();
+    AddrArifm();
+#if !__TBB_EXCEPTION_HANDLING_TOTALLY_BROKEN
+    NULLReturn(1*MByte,100*MByte,total_threads);
 #endif
-
-    // Long tests, so running in serial only. TODO - rework to make shorter
-#if __APPLE__
-    /* TODO:  check why skipped on Mac OS X */
-#else
-    test.UniquePointer();
-    test.AddrArifm();
 #endif
-
-#if __linux__
-    /* According to man pthreads 
-       "NPTL threads do not share resource limits (fixed in kernel 2.6.10)".
-       Use per-threads limits for affected systems.
-     */
-    if ( LinuxKernelVersion() < 2*1000000 + 6*1000 + 10)
-        perProcessLimits = false;
-#endif    
-
-    // Tests running in parallel
-    for( int p=MaxThread; p>=MinThread; --p ) {
-        if( Verbose )
-            printf("testing with %d threads\n", p );
-        NativeParallelFor( p, RoundRobin(p, Verbose) );
-    }
-
-    if( !error_occurred ) printf("done\n");
-    return 0;
+    if (FullLog) REPORT("All tests ended\nclearing memory...");
 }
